# -*- coding: utf-8 -*-
#
#  SelfTest/Cipher/XOR.py: Self-test for the XOR "cipher"
#
# =======================================================================
# Copyright (C) 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# =======================================================================
#

"""Self-test suite for CryptoPlus.Cipher.XOR"""

__revision__ = "$Id$"

# This is a list of (plaintext, ciphertext, key) tuples.
test_data = [
    # Test vectors written from scratch.  (Nobody posts XOR test vectors on the web?  How disappointing.)
    (b'01', b'01',
        b'00',
        'zero key'),

    (b'0102040810204080', b'0003050911214181',
        b'01',
        '1-byte key'),

    (b'0102040810204080', b'cda8c8a2dc8a8c2a',
        b'ccaa',
        '2-byte key'),

    (b'ff'*64, b'fffefdfcfbfaf9f8f7f6f5f4f3f2f1f0efeeedecebeae9e8e7e6e5e4e3e2e1e0'*2,
        b'000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f',
        '32-byte key'),

    # XOR truncates at 32 bytes in Python 2.x but not Python 3.x
#    (b'ff'*64, b'fffefdfcfbfaf9f8f7f6f5f4f3f2f1f0efeeedecebeae9e8e7e6e5e4e3e2e1e0'*2,
#        b'000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f55',
#        '33-byte key (truncated to 32 bytes)'),
]

def get_tests():
    from CryptoPlus.Cipher import XOR
    from common import make_stream_tests
    return make_stream_tests(XOR, "XOR", test_data)

if __name__ == '__main__':
    import unittest
    suite = lambda: unittest.TestSuite(get_tests())
    unittest.main(defaultTest='suite')

# vim:set ts=4 sw=4 sts=4 expandtab:



#!/usr/bin/env python

import codecs

from pkg_resources import require
require("CryptoPlus>=1.0")
from CryptoPlus.testvectors import dict_ofb_aes, dict_ctr_aes, dict_cfb_aes, dict_cbc_aes
from CryptoPlus.testvectors import dict_cmac_aes128,dict_cmac_aes192,dict_cmac_aes256,dict_cmac_tdes2,dict_cmac_tdes3
from CryptoPlus.testvectors import dict_des,dict_tdes2,dict_tdes3
from CryptoPlus.testvectors import dict_serpent128,dict_serpent192,dict_serpent256
from CryptoPlus.testvectors import dict_xts_aes
from CryptoPlus.testvectors import sha512_all_zero_messages, radiogatun32, radiogatun64

## HASHING

# SHA-512
print("SHA-512")

from CryptoPlus.Hash import python_SHA512

for i in range(0,len(sha512_all_zero_messages)):
    hash = sha512_all_zero_messages[i]
    hasher = python_SHA512.new(i*b"\x00")
    if hash != hasher.hexdigest().upper():
        print('ERROR! SHA-512 in %i'%i)

# RadioGatun
print("RadioGatun")

from CryptoPlus.Hash import python_RadioGatun

for i in range(0,len(radiogatun32)//2):
    msg = radiogatun32["msg%i"%i]
    hash = radiogatun32["hash%i"%i]
    hasher = python_RadioGatun.new(msg,wl=32)
    if hash != hasher.hexdigest().upper():
        print('ERROR! RadioGatun[32] in %i'%i)

for i in range(0,len(radiogatun64)//2):
    msg = radiogatun64["msg%i"%i]
    hash = radiogatun64["hash%i"%i]
    hasher = python_RadioGatun.new(msg,wl=64)
    if hash != hasher.hexdigest().upper():
        print('ERROR! RadioGatun[64] in %i'%i)

## CIPHERS

# PRESENT
print("PRESENT")

from CryptoPlus.testvectors import dict_present_e80_k12_tvar, dict_present_e128_k12_tvar, dict_present_e128_kvar_t12, dict_present_e80_kvar_t12
from CryptoPlus.Cipher import python_PRESENT

for i in range(1,len(dict_present_e80_k12_tvar)//3):
    msg = codecs.decode(dict_present_e80_k12_tvar['msg%i'%i], 'hex')
    key = codecs.decode(dict_present_e80_k12_tvar['key%i'%i], 'hex')
    cip = codecs.decode(dict_present_e80_k12_tvar['cip%i'%i], 'hex')
    cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    decipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    if cip != cipher.encrypt(msg):
        print('ERROR! for present_e80-k12_tvar in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for present_e80-k12_tvar in %i'%i)

for i in range(1,len(dict_present_e128_k12_tvar)//3):
    msg = codecs.decode(dict_present_e128_k12_tvar['msg%i'%i], 'hex')
    key = codecs.decode(dict_present_e128_k12_tvar['key%i'%i], 'hex')
    cip = codecs.decode(dict_present_e128_k12_tvar['cip%i'%i], 'hex')
    cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    decipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    if cip != cipher.encrypt(msg):
        print('ERROR! for present_e128-k12_tvar in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for present_e128-k12_tvar in %i'%i)

for i in range(1,len(dict_present_e128_kvar_t12)//3):
    msg = codecs.decode(dict_present_e128_kvar_t12['msg%i'%i], 'hex')
    key = codecs.decode(dict_present_e128_kvar_t12['key%i'%i], 'hex')
    cip = codecs.decode(dict_present_e128_kvar_t12['cip%i'%i], 'hex')
    cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    decipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    if cip != cipher.encrypt(msg):
        print('ERROR! for present_e128-kvar_t12 in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for present_e128-kvar_t12 in %i'%i)

for i in range(1,len(dict_present_e80_kvar_t12)//3):
    msg = codecs.decode(dict_present_e80_kvar_t12['msg%i'%i], 'hex')
    key = codecs.decode(dict_present_e80_kvar_t12['key%i'%i], 'hex')
    cip = codecs.decode(dict_present_e80_kvar_t12['cip%i'%i], 'hex')
    cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    decipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
    if cip != cipher.encrypt(msg):
        print('ERROR! for present_e80-kvar_t12 in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for present_e80-kvar_t12 in %i'%i)

# CBC, CFB, OFB and CTR with AES
print("AES")

from CryptoPlus.Cipher import python_AES
from CryptoPlus.Util import util

for i in range(1,len(dict_cbc_aes)//4+1):
    msg = codecs.decode(dict_cbc_aes['msg%i'%i], 'hex')
    iv = codecs.decode(dict_cbc_aes['iv%i'%i], 'hex')
    key = codecs.decode(dict_cbc_aes['key%i'%i], 'hex')
    cip = codecs.decode(dict_cbc_aes['cip%i'%i], 'hex')
    cipher = python_AES.new(key,python_AES.MODE_CBC,iv)
    decipher = python_AES.new(key,python_AES.MODE_CBC,iv)
    if cip != cipher.encrypt(msg):
        print('ERROR! for CBC-AES in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for CBC-AES in %i'%i)

for i in range(1,len(dict_ctr_aes)//4+1):
    msg = codecs.decode(dict_ctr_aes['msg%i'%i], 'hex')
    ctr = codecs.decode(dict_ctr_aes['ctr%i'%i], 'hex')
    key = codecs.decode(dict_ctr_aes['key%i'%i], 'hex')
    cip = codecs.decode(dict_ctr_aes['cip%i'%i], 'hex')
    counter = util.Counter(ctr)
    counter2= util.Counter(ctr)
    cipher = python_AES.new(key,python_AES.MODE_CTR,counter=counter)
    decipher = python_AES.new(key,python_AES.MODE_CTR,counter=counter2)
    if cip != cipher.encrypt(msg):
        print('ERROR! for CTR-AES in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for CTR-AES in %i'%i)

for i in range(1,len(dict_ofb_aes)//4+1):
    msg = codecs.decode(dict_ofb_aes['msg%i'%i], 'hex')
    iv = codecs.decode(dict_ofb_aes['iv%i'%i], 'hex')
    key = codecs.decode(dict_ofb_aes['key%i'%i], 'hex')
    cip = codecs.decode(dict_ofb_aes['cip%i'%i], 'hex')
    cipher = python_AES.new(key,python_AES.MODE_OFB,IV=iv)
    decipher = python_AES.new(key,python_AES.MODE_OFB,IV=iv)
    if cip != cipher.encrypt(msg):
        print('ERROR! for OFB-AES in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for OFB-AES in %i'%i)

for i in range(1,len(dict_cfb_aes)//4+1):
    msg = codecs.decode(dict_cfb_aes['msg%i'%i], 'hex')
    iv = codecs.decode(dict_cfb_aes['iv%i'%i], 'hex')
    s = dict_cfb_aes['s%i'%i]
    key = codecs.decode(dict_cfb_aes['key%i'%i], 'hex')
    cip = codecs.decode(dict_cfb_aes['cip%i'%i], 'hex')
    cipher = python_AES.new(key,python_AES.MODE_CFB,IV=iv,segment_size=s)
    decipher = python_AES.new(key,python_AES.MODE_CFB,IV=iv,segment_size=s)
    if cip != cipher.encrypt(msg):
        print('ERROR! for CFB-AES in %i'%i)
    if msg != decipher.decrypt(cip):
        print('DECRYPTION ERROR! for CFB-AES in %i'%i)

# DES,TDES2/3
print("DES TDES2/3")

from CryptoPlus.Cipher import python_DES

for i in range(0,len(dict_des)//3):
    msg = codecs.decode(dict_des['msg%i'%i], 'hex')
    key = codecs.decode(dict_des['key%i'%i], 'hex')
    cip = codecs.decode(dict_des['cip%i'%i], 'hex')
    cipher = python_DES.new(key,python_DES.MODE_ECB)
    if cip != cipher.encrypt(msg):
        print('ERROR! for DES in %i'%i)
    if msg != cipher.decrypt(cip):
        print('DECRYPTION ERROR! for DES in %i'%i)

from CryptoPlus.Cipher import python_DES3

for d in dict_tdes2,dict_tdes3:
    for i in range(0,len(d)//3):
        msg = codecs.decode(d['msg%i'%i], 'hex')
        key = codecs.decode(d['key%i'%i], 'hex')
        cip = codecs.decode(d['cip%i'%i], 'hex')
        cipher = python_DES3.new(key,python_DES3.MODE_ECB)
        if cip != cipher.encrypt(msg):
            print('ERROR! for TDES2/3 in %i'%i)
        if msg != cipher.decrypt(cip):
            print('DECRYPTION ERROR! for DES in %i'%i)

# Serpent128/192/256
print("Serpent")

from CryptoPlus.Cipher import python_Serpent

for d in dict_serpent128,dict_serpent192,dict_serpent256:
    for i in range(0,len(d)//3):
        msg = codecs.decode(d['msg%i'%i], 'hex')
        key = codecs.decode(d['key%i'%i], 'hex')
        cip = codecs.decode(d['cip%i'%i], 'hex')
        cipher = python_Serpent.new(key,python_Serpent.MODE_ECB)
        if cip != cipher.encrypt(msg):
            print('ERROR! for Serpent in %i'%i)
        if msg != cipher.decrypt(cip):
            print('DECRYPTION ERROR! for Serpent in %i'%i)

# CMAC-AES128/192/256
print("CMAC-AES")

from CryptoPlus.Cipher import python_AES

for d in dict_cmac_aes128,dict_cmac_aes192,dict_cmac_aes256:
    for i in range(0,len(d)//4):
        msg = codecs.decode(d['msg%i'%i], 'hex')
        key = codecs.decode(d['key%i'%i], 'hex')
        if msg == b'\x00':
            msg = b''
        mac = codecs.decode(d['mac%i'%i], 'hex')
        cipher = python_AES.new(key,python_AES.MODE_CMAC)
        if mac != cipher.encrypt(msg)[:d['taglength%i'%i]]:
            print('ERROR for %i'%i)

# CMAC-TDES2/3
print("CMAC-TDES")
from CryptoPlus.Cipher import python_DES3

for d in dict_cmac_tdes2,dict_cmac_tdes3:
    for i in range(0,len(d)//4):
        msg = codecs.decode(d['msg%i'%i], 'hex')
        if msg == b'\x00':
            msg = b''
        key = codecs.decode(d['key%i'%i], 'hex')
        mac = codecs.decode(d['mac%i'%i], 'hex')
        cipher = python_DES3.new(key,python_DES3.MODE_CMAC)
        if mac != cipher.encrypt(msg)[:d['taglength%i'%i]]:
            print('ERROR! on %i'%i)

# XTS-AES
print("XTS-AES")

from CryptoPlus.Cipher import python_AES

for i in range(0,len(dict_xts_aes)//5):
    msg = codecs.decode(dict_xts_aes['msg%i'%i], 'hex')
    key = ( codecs.decode(dict_xts_aes['key1_%i'%i], 'hex') , codecs.decode(dict_xts_aes['key2_%i'%i], 'hex') )
    cip = codecs.decode(dict_xts_aes['cip%i'%i], 'hex')
    n   = codecs.decode(dict_xts_aes['n%i'%i], 'hex')
    cipher = python_AES.new(key,python_AES.MODE_XTS)
    if cip != cipher.encrypt(msg,n):
        print('ERROR! for XTS on %i'%i)
        print('got %s \n expected %s'%(cipher.encrypt(msg,n).encode('hex'),cip.encode('hex')))
    decipher = python_AES.new(key,python_AES.MODE_XTS)
    if msg != cipher.decrypt(cip,n):
        print('ERROR! for XTS (decrypt) on %i'%i)
        print('got %s \n expected %s'%(decipher.decrypt(cip,n).encode('hex'),msg.encode('hex')))

# TWOFISH
print("Twofish")

from CryptoPlus.Cipher import python_Twofish
from CryptoPlus.testvectors import dict_twofish_ecb_vt_k128, dict_twofish_ecb_vt_k192, dict_twofish_ecb_vt_k256
from CryptoPlus.testvectors import dict_twofish_ecb_vk_k128, dict_twofish_ecb_vk_k192, dict_twofish_ecb_vk_k256

for d in dict_twofish_ecb_vt_k128, dict_twofish_ecb_vt_k192, dict_twofish_ecb_vt_k256,dict_twofish_ecb_vk_k128:
 for i in range(0,len(d)//3):
    msg = codecs.decode(d['msg%i'%i], 'hex')
    key = codecs.decode(d['key%i'%i], 'hex')
    cip = codecs.decode(d['cip%i'%i], 'hex')
    cipher = python_Twofish.new(key,python_Twofish.MODE_ECB)
    if cip != cipher.encrypt(msg,n):
        print('ERROR! for Twofish on %i'%i)
        print('got %s \n expected %s'%(cipher.encrypt(msg,n).encode('hex'),cip.encode('hex')))
    decipher = python_Twofish.new(key,python_AES.MODE_ECB)
    if msg != cipher.decrypt(cip,n):
        print('DECRYPTION ERROR! for Twofish (decrypt) on %i'%i)
        print('got %s \n expected %s'%(decipher.decrypt(cip,n).encode('hex'),msg.encode('hex')))



from .blockcipher import *
from .pypresent import Present

def new(key,mode=MODE_ECB,IV=None,counter=None,segment_size=None,rounds=32):
    """Create a new cipher object

    Wrapper for pure python implementation rijndael.py

        key = raw string containing the key, AES-128..256 will be selected according to the key length
        mode = python_PRESENT.MODE_ECB/CBC/CFB/OFB/CTR/CMAC, default is ECB
            -> for every mode, except ECB and CTR, it is important to construct a seperate cipher for encryption and decryption
        IV = IV as a raw string, default is "all zero" IV
            -> needed for CBC, CFB and OFB mode
        counter = counter object (CryptoPlus.Util.util.Counter)
            -> only needed for CTR mode
            -> use a seperate counter object for the cipher and decipher: the counter is updated directly, not a copy
                see CTR example further on in the docstring
                rounds = amount of rounds
        segment_size = amount of bits to use from the keystream in each chain part
            -> supported values: multiple of 8 between 8 and the blocksize
               of the cipher (only per byte access possible), default is 8
            -> only needed for CFB mode
        rounds = amount of rounds, default = 32

    Notes:
        - Always construct a seperate cipher object for encryption and decryption. Once a cipher object has been used for encryption,
          it can't be used for decryption because it keeps a state (if necessary) for the IV.

        EXAMPLES:
        **********
        IMPORTING:
        -----------
        >>> import codecs
        >>> from CryptoPlus.Cipher import python_PRESENT

        ECB Test Vectors:
        ------------------
        >>> key = codecs.decode("00000000000000000000", 'hex')
        >>> plain = codecs.decode("0000000000000000", 'hex')
        >>> cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB)
        >>> codecs.encode(cipher.encrypt(plain), 'hex')
        b'5579c1387b228445'
        
        >>> key = codecs.decode("00000000000000000000000000000000", 'hex')
        >>> plain = codecs.decode("0000000000000000", 'hex')
        >>> cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB,rounds=64)
        >>> codecs.encode(cipher.encrypt(plain), 'hex')
        b'59a27d01607ebf05'
        
        >>> key = codecs.decode("00000000000000000000", 'hex')
        >>> plain = codecs.decode("0000000000000000", 'hex')
        >>> cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB,rounds=64)
        >>> codecs.encode(cipher.encrypt(plain), 'hex')
        b'13991dd588bc1288'
        
        Test Vectors for maximum rounds supported by PRESENT reference C code:
        -----------------------------------------------------------------------
        >>> key = codecs.decode("0123456789abcdef0123", 'hex')
        >>> plain = codecs.decode("0123456789abcdef", 'hex')
        >>> cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB,rounds=65534)
        >>> ciphertext = cipher.encrypt(plain)
        >>> codecs.encode(ciphertext, 'hex')
        b'a140dc5d7175ca20'
        >>> codecs.encode(cipher.decrypt(ciphertext), 'hex')
        b'0123456789abcdef'
        
        >>> key = codecs.decode("0123456789abcdef0123456789abcdef", 'hex')
        >>> plain = codecs.decode("0123456789abcdef", 'hex')
        >>> cipher = python_PRESENT.new(key,python_PRESENT.MODE_ECB,rounds=65534)
        >>> ciphertext = cipher.encrypt(plain)
        >>> codecs.encode(ciphertext, 'hex')
        b'21007772e5d4ef14'
        >>> codecs.encode(cipher.decrypt(ciphertext), 'hex')
        b'0123456789abcdef'
        """
    return python_PRESENT(key,mode,IV,counter,rounds,segment_size)

class python_PRESENT(BlockCipher):
    key_error_message = "Key should be 80 or 128 bits"

    def __init__(self,key,mode,IV,counter,rounds,segment_size):
        cipher_module = Present
        args = {'rounds':rounds}
        self.blocksize = 8
        BlockCipher.__init__(self,key,mode,IV,counter,cipher_module,segment_size,args)

    def keylen_valid(self,key):
        return len(key) in (10,16)

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



import binascii
def number2string(i):
    """Convert a number to a string

    Input: long or integer
    Output: string (big-endian)
    """
    s=hex(i)[2:].rstrip('L')
    if len(s) % 2:
        s = '0' + s
    return binascii.unhexlify(s)

def number2string_N(i, N):
    """Convert a number to a string of fixed size

    i: long or integer
    N: length of string
    Output: string (big-endian)
    """
    s = '%0*x' % (N*2, i)
    return binascii.unhexlify(s)

def string2number(i):
    """ Convert a string to a number

    Input: string (big-endian)
    Output: long or integer
    """
    return int(binascii.hexlify(i),16)

def xorstring(a,b):
    """XOR two strings of same length

    For more complex cases, see CryptoPlus.Cipher.XOR"""
    assert len(a) == len(b)
    return number2string_N(string2number(a)^string2number(b), len(a))

class Counter(str):
    #found here: http://www.lag.net/pipermail/paramiko/2008-February.txt
    """Necessary for CTR chaining mode

    Initializing a counter object (ctr = Counter('xxx'), gives a value to the counter object.
    Everytime the object is called ( ctr() ) it returns the current value and increments it by 1.
    Input/output is a raw string.

    Counter value is big endian"""
    def __init__(self, initial_ctr):
        if not isinstance(initial_ctr, bytes):
            raise TypeError("nonce must be bytes")
        self.c = int(binascii.hexlify(initial_ctr), 16)
    def __call__(self):
        # This might be slow, but it works as a demonstration
        ctr = binascii.unhexlify("%032x" % (self.c,))
        self.c += 1
        return ctr




from pkg_resources import parse_version
import Crypto
__all__ = ["Cipher","PublicKey","Util","Protocol","Hash","testvectors","SelfTest"]

if parse_version(Crypto.__version__) > parse_version("2.0.1"):
        __all__.append("Random")

#del parse_version
#del Crypto



from . import pymd5

__all__ = ['new','digest_size']

def new(data=None):
        """Create a new pure python MD5 hash object
        
        data =  initial input (raw string) to the hashing object
                if present, the method call update(arg) is made
        
        EXAMPLE: (http://www.rfc-editor.org/rfc/rfc1321.txt)
        =========
        
        >>> from CryptoPlus.Hash import MD5
        
        >>> message = b"abc"
        >>> hasher = MD5.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        '900150983cd24fb0d6963f7d28e17f72'
        
        >>> message = b"message digest"
        >>> hasher = MD5.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        'f96b697d7cb7938d525a2f31aaf161d0'
        """
        return pymd5.new(data)
        
digest_size = pymd5.digest_size



# =============================================================================
# Copyright (c) 2008
#     Christophe Oosterlynck (christophe.oosterlynck_AT_gmail.com)
#     Philippe Teuwen (philippe.teuwen_AT_nxp.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# =============================================================================

"""RadioGatun pure python implementation

Code based on the standard found here: http://radiogatun.noekeon.org/
Api and code interface is based on the MD5 implementation of pypy
 http://codespeak.net/pypy/dist/pypy/doc/home.html
"""

BELT_WIDTH = 3
BELT_LENGTH = 13
MILL_SIZE = 2*BELT_WIDTH + BELT_LENGTH
NUMBER_OF_BLANK_ITERATIONS = 16

def state_init():
    """construct an empty state variable
    """
    return {"A":[0]*MILL_SIZE, "B":[[0]*BELT_WIDTH for x in range(BELT_LENGTH)]}

def XOR_F_i(state, inp, wl):
    """Input mapping

    mapping input blocks to a state variable + XOR step of the alternating-
    input construction

    input = 1 blocklength string
    wl    = wordlength of the RadioGatun hash object
    """
    for i in range(BELT_WIDTH):
        # reverse endianness of byte ordering and convert the input
        #  block to integer
        p_i = string2number(inp[i*wl:(i+1)*wl][::-1])
        state["B"][0][i] ^= p_i
        state["A"][i+MILL_SIZE-BELT_WIDTH] ^= p_i
    return state

def R(state, wl):
    """Round function R

    state = the RadioGatun status
    wl    = wordlength of the RadioGatun hash object
    """
    out = state_init()
    # Belt function: simple rotation
    out["B"] = state["B"][-1:]+state["B"][:-1]
    # Mill to belt feedforward
    for i in range(BELT_LENGTH - 1):
        out["B"][i+1][i%BELT_WIDTH] ^= state["A"][i+1]
    # Run the mill
    out["A"] = Mill(state["A"], wl)
    # Belt to mill feedforward
    for i in range(BELT_WIDTH):
        out["A"][i+BELT_LENGTH] ^= state["B"][-1][i]
    return out

def Mill(a, wl):
    """The Mill function

    a  = Mill variable of the RadioGatun status
    wl = wordlength of the RadioGatun hash object
    """
    A = [0]*MILL_SIZE
    # Gamma: Non-linearity
    for i in range(MILL_SIZE):
        A[i] = a[i] ^ ~((~a[(i+1)%MILL_SIZE]) & (a[(i+2)%MILL_SIZE]) )
    # Pi: Intra-word and inter-word dispersion
    for i in range(MILL_SIZE):
        a[i] = rotateRight(A[(7*i)%MILL_SIZE], i*(i+1)//2, wl*8)
    # Theta: Diffusion
    for i in range(MILL_SIZE):
        A[i] = a[i] ^ a[(i+1)%MILL_SIZE] ^ a[(i+4)%MILL_SIZE]
    # Iota: Asymmetry
    A[0] = A[0] ^ 1
    return A

class RadioGatunType:
    "An implementation of the RadioGatun hash function in pure Python."

    def __init__(self, wl):
        """Initialisation.
        
        wl = wordlength (in bits) of the RadioGatun hash method
              between 8 and 64 (default = 64)
        """

        if not ( 8 <= wl <= 64) or not (wl%8 == 0 ):
            raise ValueError("Wordlength should be a multiple of 8" +\
                              " between 8 and 64")

        # word & block length in bytes
        self.wordlength = wl//8
        self.blocklength = self.wordlength*BELT_WIDTH
        
        # Initial message length in bits(!).
        self.length = 0
        self.count = 0

        # Initial empty message as a sequence of bytes (8 bit characters).
        self.input = b""

        # Call a separate init function, that can be used repeatedly
        # to start from scratch on the same object.
        self.init()


    def init(self):
        """Initialize the message-digest and set all fields to zero.

        Can be used to reinitialize the hash object
        """

        self.S = state_init()

        self.length = 0
        self.count = 0
        self.input = b""

    def _transform(self, inp):
        """Basic RadioGatun step transforming the digest based on the input.

        Performs the inside of the first loop of alternating input construction
        of RadioGatun. The only thing that can be done every time new data is
        submitted to the hash object.
        Mangling and output mapping can only follow when all input data has
        been received.
        """
        T = XOR_F_i(self.S, inp, self.wordlength)
        self.S = R(T, self.wordlength)


    # Down from here all methods follow the Python Standard Library
    # API of the md5 module.

    def update(self, inBuf):
        """Add to the current message.

        Update the radiogatun object with the string arg. Repeated calls
        are equivalent to a single call with the concatenation of all
        the arguments, i.e. m.update(a); m.update(b) is equivalent
        to m.update(a+b).

        The hash is immediately calculated for all full blocks. The final
        calculation is made in digest(). This allows us to keep an
        intermediate value for the hash, so that we only need to make
        minimal recalculation if we call update() to add moredata to
        the hashed string.
        """
        # Amount of bytes given at input
        leninBuf = int(len(inBuf))

        # Compute number of bytes mod 64.
        index = (self.count >> 3) % self.blocklength

        # Update number of bits.
        self.count = self.count + (leninBuf << 3)

        partLen = self.blocklength - index

        # if length of input is at least
        # the amount of bytes needed to fill a block
        if leninBuf >= partLen:
            self.input = self.input[:index] + inBuf[:partLen]
            self._transform(self.input)
            i = partLen
            while i + self.blocklength - 1 < leninBuf:
                self._transform(inBuf[i:i+self.blocklength])
                i = i + self.blocklength
            else:
                self.input = inBuf[i:leninBuf]
        # if not enough bytes at input to fill a block
        else:
            i = 0
            self.input = self.input + inBuf


    def digest(self, length=256):
        """Terminate the message-digest computation and return digest.

        length = output length of the digest in bits 
                  any multiple of 8 with a minimum of 8
                  default = 256

        Return the digest of the strings passed to the update()
        method so far. This is a byte string which may contain
        non-ASCII characters, including null bytes.
        
        Calling digest() doesn't change the internal state. Adding data via
        update() can still continu after asking for an intermediate digest
        value.
        """

        S = self.S
        inp = b"" + self.input
        count = self.count

        index = (self.count >> 3) % self.blocklength

        padLen = self.blocklength - index

        padding = [b'\001'] + [b'\000'] * (padLen - 1)
        self.update(b''.join(padding))

        # Mangling = blank rounds
        for i in range(NUMBER_OF_BLANK_ITERATIONS):
            self.S = R(self.S, self.wordlength)

        # Extraction
        # Store state in digest.
        digest = b""
        for i in range((length)//self.wordlength//2):
            self.S = R(self.S, self.wordlength)
            # F_o
            digest += \
                number2string_N((self.S["A"][1]), self.wordlength)[::-1] +\
                number2string_N((self.S["A"][2]), self.wordlength)[::-1]
 
        self.S = S
        self.input = inp
        self.count = count

        return digest[:length//8]


    def hexdigest(self, length=256):
        """Terminate and return digest in HEX form.

        length = output length of the digest in bits 
                  any multiple of 8 with a minimum of 8
                  default = 256

        Like digest() except the digest is returned as a string of
        length 'length', containing only hexadecimal digits. This may be
        used to exchange the value safely in email or other non-
        binary environments.

        Calling hexdigest() doesn't change the internal state. Adding data via
        update() can still continu after asking for an intermediate digest
        value.
        """

        import codecs
        return "{}".format(codecs.encode(self.digest(length), 'hex').decode('ascii'))

    def copy(self):
        """Return a clone object.

        Return a copy ('clone') of the radiogatun object. This can be used
        to efficiently compute the digests of strings that share
        a common initial substring.
        """

        import copy
        return copy.deepcopy(self)

# ======================================================================
# TOP LEVEL INTERFACE
# ======================================================================

def new(arg=None, wl=64):
    """Return a new RadioGatun hash object

    wl  = wordlength (in bits) of the RadioGatun hash method
              between 1 and 64 (default = 64)
    arg =  if present, the method call update(arg) is made

    EXAMPLES: (testvectors from: http://radiogatun.noekeon.org/)
    ==========
    >>> import pyradiogatun
    
    radiogatun[64]
    ---------------
    >>> hasher = pyradiogatun.new()
    >>> hasher.update(b'1234567890123456')
    >>> hasher.hexdigest()
    'caaec14b5b4a7960d6854709770e3071d635d60224f58aa385867e549ef4cc42'

    >>> hasher = pyradiogatun.new()
    >>> hasher.update(b'Santa Barbara, California')
    >>> hasher.hexdigest(480)
    '0d08daf2354fa95aaa5b6a50f514384ecdd35940252e0631002e600e13cd285f74adb0c0a666adeb1f2d20b1f2489e3d973dae4efc1f2cc5aaa13f2b'
    
    radiogatun[32]
    ---------------
    >>> hasher = pyradiogatun.new(wl=32)
    >>> hasher.update(b'1234567890123456')
    >>> hasher.hexdigest()
    '59612324f3f42d3096e69125d2733b86143ae668ae9ed561ad785e0eac8dba25'

    >>> hasher = pyradiogatun.new(wl=32)
    >>> hasher.update(b'Santa Barbara, California')
    >>> hasher.hexdigest(512)
    '041666388ef9655d48996a66dada1193d6646012a7b25a24fb10e6075cf0fc54a162949f4022531dbb6f66b64c3579df49f0f3af5951df9d68af310f2703b06d'

    radiogatun[16]
    ---------------
    >>> hasher = pyradiogatun.new(wl=16)
    >>> hasher.update(b'Santa Barbara, California')
    >>> hasher.hexdigest()
    'ab2203a8c3de943309b685513a29060339c001acce5900dcd6427a02c1fb8011'

    radiogatun[8]
    --------------
    >>> hasher = pyradiogatun.new(wl=8)
    >>> hasher.update(b'Santa Barbara, California')
    >>> hasher.hexdigest()
    'e08f5cdbbfd8f5f3c479464a60ac186963e741d28f654e2c961d2f9bebc7de31'
    """

    crypto = RadioGatunType(wl)
    if arg:
        crypto.update(arg)

    return crypto

# ======================================================================
# HELPER FUNCTIONS
# ======================================================================

def rotateRight(x, amountToShift, totalBits):
    """Rotate x (consisting of 'totalBits' bits) n bits to right.

    x             = integer input to be rotated
    amountToShift = the amount of bits that should be shifted
    totalBits     = total amount bits at the input for rotation
    """
    x = x%(2**totalBits)
    n_mod = ((amountToShift % totalBits) + totalBits) % totalBits
    return  ((x >> n_mod) | ((x << (totalBits-n_mod)))&((2**totalBits)-1))

def string2number(i):
    """ Convert a string to a number

    Input: string (big-endian)
    Output: long or integer
    """
    import codecs
    return int(codecs.encode(i, 'hex'), 16)

def number2string_N(i, N):
    """Convert a number to a string of fixed size

    i: long or integer
    N: length of string
    Output: string (big-endian)
    """
    s = '%0*x' % (N*2, i)
    import codecs
    return codecs.decode(s, 'hex')

# ======================================================================
# DOCTEST ENABLER
# ======================================================================

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    print("DOCTEST running... no messages = all good")
    _test()



from __future__ import absolute_import
from .blockcipher import *
import Crypto.Cipher.CAST

def new(key,mode=MODE_ECB,IV=None,counter=None,segment_size=None):
    """Create a new cipher object

    CAST using pycrypto for algo and pycryptoplus for ciphermode

        key = raw string containing the keys
        mode = python_AES.MODE_ECB/CBC/CFB/OFB/CTR/CMAC, default is ECB
        IV = IV as a raw string, default is "all zero" IV
            -> only needed for CBC mode
        counter = counter object (CryptoPlus.Util.util.Counter)
            -> only needed for CTR mode
        segment_size = amount of bits to use from the keystream in each chain part
            -> supported values: multiple of 8 between 8 and the blocksize
               of the cipher (only per byte access possible), default is 8
            -> only needed for CFB mode

    EXAMPLES:
    **********
    IMPORTING:
    -----------
    >>> import codecs
    >>> from CryptoPlus.Cipher import CAST

    ECB example: http://www.rfc-editor.org/rfc/rfc2144.txt
    -------------
    128 bit key

    >>> key = codecs.decode("0123456712345678234567893456789A", 'hex')
    >>> plaintext = codecs.decode("0123456789ABCDEF", 'hex')
    >>> cipher = CAST.new(key,CAST.MODE_ECB,)
    >>> codecs.encode(cipher.encrypt(plaintext), 'hex')
    b'238b4fe5847e44b2'

    40 bit key
    >>> from CryptoPlus.Cipher import CAST
    >>> key = codecs.decode("0123456712", 'hex')
    >>> plaintext = codecs.decode("0123456789ABCDEF", 'hex')
    >>> cipher = CAST.new(key,CAST.MODE_ECB,)
    >>> codecs.encode(cipher.encrypt(plaintext), 'hex').upper()
    b'7AC816D16E9B302E'
    """
    return CAST(key,mode,IV,counter,segment_size)

class CAST(BlockCipher):
    def __init__(self,key,mode,IV,counter,segment_size):
        cipher_module = Crypto.Cipher.CAST.new
        self.blocksize = 8
        BlockCipher.__init__(self,key,mode,IV,counter,cipher_module,segment_size)

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from __future__ import absolute_import
from .blockcipher import *
try:
    import Crypto.Cipher.RC5
except ImportError:
    print("Crypto.Cipher.RC5 isn't available. You're probably using the Debian pycrypto version. Install the original pycrypto for RC5.")
    raise

def new(key,mode=MODE_ECB,IV=None,counter=None,segment_size=None,rounds=12,word_size=32):
    """Create a new cipher object

    RC5 using pycrypto for algo and pycryptoplus for ciphermode

        key = raw string containing the keys
              multiple of 8 bits between 0 <-> 2040 bits
        mode = python_AES.MODE_ECB/CBC/CFB/OFB/CTR/CMAC, default is ECB
        IV = IV as a raw string, default is "all zero" IV
            -> only needed for CBC mode
        counter = counter object (CryptoPlus.Util.util.Counter)
            -> only needed for CTR mode
        segment_size = amount of bits to use from the keystream in each chain part
            -> supported values: multiple of 8 between 8 and the blocksize
               of the cipher (only per byte access possible), default is 8
            -> only needed for CFB mode
        rounds = amount of rounds, default = 12
                 minimum 12 and multiple of 2
        word_size = RC5 word size (bits), supported = 16 and 32, default = 32
                    RC5 encrypts blocks of size 2*word_size

    EXAMPLES:
    **********
    IMPORTING:
    -----------
    >>> from CryptoPlus.Cipher import RC5

    https://www.cosic.esat.kuleuven.be/nessie/testvectors/
    -----------------------------------------
    >>> key = "00000000000000000000000000000000".decode('hex')
    >>> plaintext = "0000000000000000".decode('hex')
    >>> rounds = 12
    >>> cipher = RC5.new(key,RC5.MODE_ECB,rounds=rounds)
    >>> cipher.encrypt(plaintext).encode('hex')
    '21a5dbee154b8f6d'
    """
    return RC5(key,mode,IV,counter,rounds,word_size,segment_size)

class RC5(BlockCipher):
    def __init__(self,key,mode,IV,counter,rounds,word_size,segment_size):
        cipher_module = Crypto.Cipher.RC5.new
        args = {'rounds':rounds,'word_size':word_size}
        self.blocksize = (2*word_size)/8
        BlockCipher.__init__(self,key,mode,IV,counter,cipher_module,segment_size,args)

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from . import pysha

__all__ = ['new','digest_size']

def new(data=None):
        """Create a new pure python SHA hash object
        
        data =  initial input (raw string) to the hashing object
                if present, the method call update(arg) is made
        
        EXAMPLE: FIPS 180-2
        =========
        
        >>> from CryptoPlus.Hash import python_SHA
        
        >>> message = b"abc"
        >>> hasher = python_SHA.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        'a9993e364706816aba3e25717850c26c9cd0d89d'
        
        >>> message = b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
        >>> hasher = python_SHA.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        '84983e441c3bd26ebaae4aa1f95129e5e54670f1'
        """
        return pysha.new(data)
        
digest_size = pysha.digest_size



#!/usr/bin/python
__author__ = 'Thomas Dixon'
__license__ = 'MIT'

from .pysha512 import sha512

def new(m=None):
    return sha384(m)

class sha384(sha512):
    _h = (0xcbbb9d5dc1059ed8, 0x629a292a367cd507, 0x9159015a3070dd17, 0x152fecd8f70e5939,
          0x67332667ffc00b31, 0x8eb44a8768581511, 0xdb0c2e0d64f98fa7, 0x47b5481dbefa4fa4)
    _output_size = 6
    digest_size = 48


# -*- coding: utf-8 -*-
#
#  SelfTest/Cipher/ARC2.py: Self-test for the Alleged-RC2 cipher
#
# =======================================================================
# Copyright (C) 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# =======================================================================
#

"""Self-test suite for CryptoPlus.Cipher.ARC2"""

__revision__ = "$Id$"

from common import dict     # For compatibility with Python 2.1 and 2.2

# This is a list of (plaintext, ciphertext, key[, description[, extra_params]]) tuples.
test_data = [
    # Test vectors from RFC 2268

    # 63-bit effective key length
    (b'0000000000000000', b'ebb773f993278eff', b'0000000000000000',
        'RFC2268-1', dict(effective_keylen=63)),

    # 64-bit effective key length
    (b'ffffffffffffffff', b'278b27e42e2f0d49', b'ffffffffffffffff',
        'RFC2268-2', dict(effective_keylen=64)),
    (b'1000000000000001', b'30649edf9be7d2c2', b'3000000000000000',
        'RFC2268-3', dict(effective_keylen=64)),
    (b'0000000000000000', b'61a8a244adacccf0', b'88',
        'RFC2268-4', dict(effective_keylen=64)),
    (b'0000000000000000', b'6ccf4308974c267f', b'88bca90e90875a',
        'RFC2268-5', dict(effective_keylen=64)),
    (b'0000000000000000', b'1a807d272bbe5db1', b'88bca90e90875a7f0f79c384627bafb2',
        'RFC2268-6', dict(effective_keylen=64)),

    # 128-bit effective key length
    (b'0000000000000000', b'2269552ab0f85ca6', b'88bca90e90875a7f0f79c384627bafb2',
        "RFC2268-7", dict(effective_keylen=128)),
    (b'0000000000000000', b'5b78d3a43dfff1f1',
        b'88bca90e90875a7f0f79c384627bafb216f80a6f85920584c42fceb0be255daf1e',
        "RFC2268-8", dict(effective_keylen=129)),

    # Test vectors from PyCryptoPlus 2.0.1's testdata.py
    # 1024-bit effective key length
    (b'0000000000000000', b'624fb3e887419e48', b'5068696c6970476c617373',
        'PCTv201-0',dict(effective_keylen=1024)),
    (b'ffffffffffffffff', b'79cadef44c4a5a85', b'5068696c6970476c617373',
        'PCTv201-1',dict(effective_keylen=1024)),
    (b'0001020304050607', b'90411525b34e4c2c', b'5068696c6970476c617373',
        'PCTv201-2',dict(effective_keylen=1024)),
    (b'0011223344556677', b'078656aaba61cbfb', b'5068696c6970476c617373',
        'PCTv201-3',dict(effective_keylen=1024)),
    (b'0000000000000000', b'd7bcc5dbb4d6e56a', b'ffffffffffffffff', 'PCTv201-4',dict(effective_keylen=1024)),
    (b'ffffffffffffffff', b'7259018ec557b357', b'ffffffffffffffff', 'PCTv201-5',dict(effective_keylen=1024)),
    (b'0001020304050607', b'93d20a497f2ccb62', b'ffffffffffffffff', 'PCTv201-6',dict(effective_keylen=1024)),
    (b'0011223344556677', b'cb15a7f819c0014d', b'ffffffffffffffff', 'PCTv201-7',dict(effective_keylen=1024)),
    (b'0000000000000000', b'63ac98cdf3843a7a',
        b'ffffffffffffffff5065746572477265656e6177617953e5ffe553',
        'PCTv201-8',dict(effective_keylen=1024)),
    (b'ffffffffffffffff', b'3fb49e2fa12371dd',
        b'ffffffffffffffff5065746572477265656e6177617953e5ffe553',
        'PCTv201-9',dict(effective_keylen=1024)),
    (b'0001020304050607', b'46414781ab387d5f',
        b'ffffffffffffffff5065746572477265656e6177617953e5ffe553',
        'PCTv201-10',dict(effective_keylen=1024)),
    (b'0011223344556677', b'be09dc81feaca271',
        b'ffffffffffffffff5065746572477265656e6177617953e5ffe553',
        'PCTv201-11',dict(effective_keylen=1024)),
    (b'0000000000000000', b'e64221e608be30ab', b'53e5ffe553', 'PCTv201-12',dict(effective_keylen=1024)),
    (b'ffffffffffffffff', b'862bc60fdcd4d9a9', b'53e5ffe553', 'PCTv201-13',dict(effective_keylen=1024)),
    (b'0001020304050607', b'6a34da50fa5e47de', b'53e5ffe553', 'PCTv201-14',dict(effective_keylen=1024)),
    (b'0011223344556677', b'584644c34503122c', b'53e5ffe553', 'PCTv201-15',dict(effective_keylen=1024)),
]

def get_tests():
    from CryptoPlus.Cipher import ARC2
    from common import make_block_tests
    return make_block_tests(ARC2, "ARC2", test_data)

if __name__ == '__main__':
    import unittest
    suite = lambda: unittest.TestSuite(get_tests())
    unittest.main(defaultTest='suite')

# vim:set ts=4 sw=4 sts=4 expandtab:



from __future__ import absolute_import
from .blockcipher import *
import Crypto.Cipher.DES

def new(key,mode=MODE_ECB,IV=None,counter=None,segment_size=None):
    """Create a new cipher object

    DES using pycrypto for algo and pycryptoplus for ciphermode

        key = raw string containing the keys
        mode = python_AES.MODE_ECB/CBC/CFB/OFB/CTR/CMAC, default is ECB
        IV = IV as a raw string, default is "all zero" IV
            -> only needed for CBC mode
        counter = counter object (CryptoPlus.Util.util.Counter)
            -> only needed for CTR mode
        segment_size = amount of bits to use from the keystream in each chain part
            -> supported values: multiple of 8 between 8 and the blocksize
               of the cipher (only per byte access possible), default is 8
            -> only needed for CFB mode

    EXAMPLES:
    **********
    IMPORTING:
    -----------
    >>> import codecs
    >>> from CryptoPlus.Cipher import DES

    EXAMPLE (test vectors from NESSIE):
    -----------------------------------

    >>> cipher = DES.new(codecs.decode('7CA110454A1A6E57', 'hex'))
    >>> ciphertext = cipher.encrypt(codecs.decode('01A1D6D039776742', 'hex'))
    >>> codecs.encode(ciphertext, 'hex')
    b'690f5b0d9a26939b'
    >>> plaintext = cipher.decrypt(ciphertext)
    >>> codecs.encode(plaintext, 'hex')
    b'01a1d6d039776742'

    """
    return DES(key,mode,IV,counter,segment_size)

class DES(BlockCipher):
    def __init__(self,key,mode,IV,counter,segment_size):
        cipher_module = Crypto.Cipher.DES.new
        self.blocksize = 8
        BlockCipher.__init__(self,key,mode,IV,counter,cipher_module,segment_size)

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



#############################################################################
#               Documentation                   #
#############################################################################

# Author:   Todd Whiteman
# Date:     12th September, 2005
# Verion:   1.2
# Homepage: http://twhiteman.netfirms.com/des.html
#
# This algorithm is a pure python implementation of the DES algorithm.
# It is in pure python to avoid portability issues, since most DES
# implementations are programmed in C (for performance reasons).
#
# Triple DES class is also implemented, utilising the DES base. Triple DES
# is either DES-EDE3 with a 24 byte key, or DES-EDE2 with a 16 byte key.
#
# See the README.txt that should come with this python module for the
# implementation methods used.
#
# Thanks to David Broadwell for ideas, comments and suggestions.
# Thanks to Mario Wolff for pointing out and debugging some triple des CBC errors.
#
"""A pure python implementation of the DES and TRIPLE DES encryption algorithms

pyDes.des(key, [mode], [IV])
pyDes.triple_des(key, [mode], [IV])

key  -> String containing the encryption key. 8 bytes for DES, 16 or 24 bytes
    for Triple DES
mode -> Optional argument for encryption type, can be either
        pyDes.ECB (Electronic Code Book) or pyDes.CBC (Cypher Block Chaining)
IV   -> Optional argument, must be supplied if using CBC mode. Must be 8 bytes


Example:
from pyDes import *

data = "Please encrypt my string"
k = des("DESCRYPT", " ", CBC, "\0\0\0\0\0\0\0\0")
d = k.encrypt(data)
print("Encypted string: " + d)
print("Decypted string: " + k.decrypt(d))

See the module source (pyDes.py) for more examples of use.
You can slo run the pyDes.py file without and arguments to see a simple test.

Note: This code was not written for high-end systems needing a fast
      implementation, but rather a handy portable solution with small usage.

"""


# Modes of crypting / cyphering
ECB =   0
CBC =   1


#############################################################################
#                   DES                     #
#############################################################################
class des:
    """DES encryption/decrytpion class

    Supports ECB (Electronic Code Book) and CBC (Cypher Block Chaining) modes.

    pyDes.des(key,[mode], [IV])

    key  -> The encryption key string, must be exactly 8 bytes
    mode -> Optional argument for encryption type, can be either pyDes.ECB
        (Electronic Code Book), pyDes.CBC (Cypher Block Chaining)
    IV   -> Optional string argument, must be supplied if using CBC mode.
        Must be 8 bytes in length.
    """


    # Permutation and translation tables for DES
    __pc1 = [56, 48, 40, 32, 24, 16,  8,
          0, 57, 49, 41, 33, 25, 17,
          9,  1, 58, 50, 42, 34, 26,
         18, 10,  2, 59, 51, 43, 35,
         62, 54, 46, 38, 30, 22, 14,
          6, 61, 53, 45, 37, 29, 21,
         13,  5, 60, 52, 44, 36, 28,
         20, 12,  4, 27, 19, 11,  3
    ]

    # number left rotations of pc1
    __left_rotations = [
        1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1
    ]

    # permuted choice key (table 2)
    __pc2 = [
        13, 16, 10, 23,  0,  4,
         2, 27, 14,  5, 20,  9,
        22, 18, 11,  3, 25,  7,
        15,  6, 26, 19, 12,  1,
        40, 51, 30, 36, 46, 54,
        29, 39, 50, 44, 32, 47,
        43, 48, 38, 55, 33, 52,
        45, 41, 49, 35, 28, 31
    ]

    # initial permutation IP
    __ip = [57, 49, 41, 33, 25, 17, 9,  1,
        59, 51, 43, 35, 27, 19, 11, 3,
        61, 53, 45, 37, 29, 21, 13, 5,
        63, 55, 47, 39, 31, 23, 15, 7,
        56, 48, 40, 32, 24, 16, 8,  0,
        58, 50, 42, 34, 26, 18, 10, 2,
        60, 52, 44, 36, 28, 20, 12, 4,
        62, 54, 46, 38, 30, 22, 14, 6
    ]

    # Expansion table for turning 32 bit blocks into 48 bits
    __expansion_table = [
        31,  0,  1,  2,  3,  4,
         3,  4,  5,  6,  7,  8,
         7,  8,  9, 10, 11, 12,
        11, 12, 13, 14, 15, 16,
        15, 16, 17, 18, 19, 20,
        19, 20, 21, 22, 23, 24,
        23, 24, 25, 26, 27, 28,
        27, 28, 29, 30, 31,  0
    ]

    # The (in)famous S-boxes
    __sbox = [
        # S1
        [14, 4, 13, 1, 2, 15, 11, 8, 3, 10, 6, 12, 5, 9, 0, 7,
         0, 15, 7, 4, 14, 2, 13, 1, 10, 6, 12, 11, 9, 5, 3, 8,
         4, 1, 14, 8, 13, 6, 2, 11, 15, 12, 9, 7, 3, 10, 5, 0,
         15, 12, 8, 2, 4, 9, 1, 7, 5, 11, 3, 14, 10, 0, 6, 13],

        # S2
        [15, 1, 8, 14, 6, 11, 3, 4, 9, 7, 2, 13, 12, 0, 5, 10,
         3, 13, 4, 7, 15, 2, 8, 14, 12, 0, 1, 10, 6, 9, 11, 5,
         0, 14, 7, 11, 10, 4, 13, 1, 5, 8, 12, 6, 9, 3, 2, 15,
         13, 8, 10, 1, 3, 15, 4, 2, 11, 6, 7, 12, 0, 5, 14, 9],

        # S3
        [10, 0, 9, 14, 6, 3, 15, 5, 1, 13, 12, 7, 11, 4, 2, 8,
         13, 7, 0, 9, 3, 4, 6, 10, 2, 8, 5, 14, 12, 11, 15, 1,
         13, 6, 4, 9, 8, 15, 3, 0, 11, 1, 2, 12, 5, 10, 14, 7,
         1, 10, 13, 0, 6, 9, 8, 7, 4, 15, 14, 3, 11, 5, 2, 12],

        # S4
        [7, 13, 14, 3, 0, 6, 9, 10, 1, 2, 8, 5, 11, 12, 4, 15,
         13, 8, 11, 5, 6, 15, 0, 3, 4, 7, 2, 12, 1, 10, 14, 9,
         10, 6, 9, 0, 12, 11, 7, 13, 15, 1, 3, 14, 5, 2, 8, 4,
         3, 15, 0, 6, 10, 1, 13, 8, 9, 4, 5, 11, 12, 7, 2, 14],

        # S5
        [2, 12, 4, 1, 7, 10, 11, 6, 8, 5, 3, 15, 13, 0, 14, 9,
         14, 11, 2, 12, 4, 7, 13, 1, 5, 0, 15, 10, 3, 9, 8, 6,
         4, 2, 1, 11, 10, 13, 7, 8, 15, 9, 12, 5, 6, 3, 0, 14,
         11, 8, 12, 7, 1, 14, 2, 13, 6, 15, 0, 9, 10, 4, 5, 3],

        # S6
        [12, 1, 10, 15, 9, 2, 6, 8, 0, 13, 3, 4, 14, 7, 5, 11,
         10, 15, 4, 2, 7, 12, 9, 5, 6, 1, 13, 14, 0, 11, 3, 8,
         9, 14, 15, 5, 2, 8, 12, 3, 7, 0, 4, 10, 1, 13, 11, 6,
         4, 3, 2, 12, 9, 5, 15, 10, 11, 14, 1, 7, 6, 0, 8, 13],

        # S7
        [4, 11, 2, 14, 15, 0, 8, 13, 3, 12, 9, 7, 5, 10, 6, 1,
         13, 0, 11, 7, 4, 9, 1, 10, 14, 3, 5, 12, 2, 15, 8, 6,
         1, 4, 11, 13, 12, 3, 7, 14, 10, 15, 6, 8, 0, 5, 9, 2,
         6, 11, 13, 8, 1, 4, 10, 7, 9, 5, 0, 15, 14, 2, 3, 12],

        # S8
        [13, 2, 8, 4, 6, 15, 11, 1, 10, 9, 3, 14, 5, 0, 12, 7,
         1, 15, 13, 8, 10, 3, 7, 4, 12, 5, 6, 11, 0, 14, 9, 2,
         7, 11, 4, 1, 9, 12, 14, 2, 0, 6, 10, 13, 15, 3, 5, 8,
         2, 1, 14, 7, 4, 10, 8, 13, 15, 12, 9, 0, 3, 5, 6, 11],
    ]


    # 32-bit permutation function P used on the output of the S-boxes
    __p = [
        15, 6, 19, 20, 28, 11,
        27, 16, 0, 14, 22, 25,
        4, 17, 30, 9, 1, 7,
        23,13, 31, 26, 2, 8,
        18, 12, 29, 5, 21, 10,
        3, 24
    ]

    # final permutation IP^-1
    __fp = [
        39,  7, 47, 15, 55, 23, 63, 31,
        38,  6, 46, 14, 54, 22, 62, 30,
        37,  5, 45, 13, 53, 21, 61, 29,
        36,  4, 44, 12, 52, 20, 60, 28,
        35,  3, 43, 11, 51, 19, 59, 27,
        34,  2, 42, 10, 50, 18, 58, 26,
        33,  1, 41,  9, 49, 17, 57, 25,
        32,  0, 40,  8, 48, 16, 56, 24
    ]

    # Type of crypting being done
    ENCRYPT =   0x00
    DECRYPT =   0x01

    # Initialisation
    def __init__(self, key, mode=ECB, IV=None):
        if len(key) != 8:
            raise ValueError("Invalid DES key size. Key must be exactly 8 bytes long.")
        self.block_size = 8
        self.key_size = 8
        self.__padding = b''

        # Set the passed in variables
        self.setMode(mode)
        if IV:
            self.setIV(IV)

        self.L = []
        self.R = []
        self.Kn = [ [0] * 48 ] * 16 # 16 48-bit keys (K1 - K16)
        self.final = []

        self.setKey(key)


    def getKey(self):
        """getKey() -> string"""
        return self.__key

    def setKey(self, key):
        """Will set the crypting key for this object. Must be 8 bytes."""
        self.__key = key
        self.__create_sub_keys()

    def getMode(self):
        """getMode() -> pyDes.ECB or pyDes.CBC"""
        return self.__mode

    def setMode(self, mode):
        """Sets the type of crypting mode, pyDes.ECB or pyDes.CBC"""
        self.__mode = mode

    def getIV(self):
        """getIV() -> string"""
        return self.__iv

    def setIV(self, IV):
        """Will set the Initial Value, used in conjunction with CBC mode"""
        if not IV or len(IV) != self.block_size:
            raise ValueError("Invalid Initial Value (IV), must be a multiple of " + str(self.block_size) + " bytes")
        self.__iv = IV

    def getPadding(self):
        """getPadding() -> string of length 1. Padding character."""
        return self.__padding

    def __String_to_BitList(self, data):
        """Turn the string data, into a list of bits (1, 0)'s"""
        l = len(data) * 8
        result = [0] * l
        pos = 0
        for c in bytearray(data):
            i = 7
            ch = c
            while i >= 0:
                if ch & (1 << i) != 0:
                    result[pos] = 1
                else:
                    result[pos] = 0
                pos += 1
                i -= 1

        return result

    def __BitList_to_String(self, data):
        """Turn the list of bits -> data, into a string"""
        result = bytearray()
        pos = 0
        c = 0
        while pos < len(data):
            c += data[pos] << (7 - (pos % 8))
            if (pos % 8) == 7:
                result.append(c)
                c = 0
            pos += 1

        return bytes(result)

    def __permutate(self, table, block):
        """Permutate this block with the specified table"""
        return [block[x] for x in table]

    # Transform the secret key, so that it is ready for data processing
    # Create the 16 subkeys, K[1] - K[16]
    def __create_sub_keys(self):
        """Create the 16 subkeys K[1] to K[16] from the given key"""
        key = self.__permutate(des.__pc1, self.__String_to_BitList(self.getKey()))
        i = 0
        # Split into Left and Right sections
        self.L = key[:28]
        self.R = key[28:]
        while i < 16:
            j = 0
            # Perform circular left shifts
            while j < des.__left_rotations[i]:
                self.L.append(self.L[0])
                del self.L[0]

                self.R.append(self.R[0])
                del self.R[0]

                j += 1

            # Create one of the 16 subkeys through pc2 permutation
            self.Kn[i] = self.__permutate(des.__pc2, self.L + self.R)

            i += 1

    # Main part of the encryption algorithm, the number cruncher :)
    def __des_crypt(self, block, crypt_type):
        """Crypt the block of data through DES bit-manipulation"""
        block = self.__permutate(des.__ip, block)
        self.L = block[:32]
        self.R = block[32:]

        # Encryption starts from Kn[1] through to Kn[16]
        if crypt_type == des.ENCRYPT:
            iteration = 0
            iteration_adjustment = 1
        # Decryption starts from Kn[16] down to Kn[1]
        else:
            iteration = 15
            iteration_adjustment = -1

        i = 0
        while i < 16:
            # Make a copy of R[i-1], this will later become L[i]
            tempR = self.R[:]

            # Permutate R[i - 1] to start creating R[i]
            self.R = self.__permutate(des.__expansion_table, self.R)

            # Exclusive or R[i - 1] with K[i], create B[1] to B[8] whilst here
            self.R = list(map(lambda x, y: x ^ y, self.R, self.Kn[iteration]))
            B = [self.R[:6], self.R[6:12], self.R[12:18], self.R[18:24], self.R[24:30], self.R[30:36], self.R[36:42], self.R[42:]]
            # Optimization: Replaced below commented code with above
            #j = 0
            #B = []
            #while j < len(self.R):
            #   self.R[j] = self.R[j] ^ self.Kn[iteration][j]
            #   j += 1
            #   if j % 6 == 0:
            #       B.append(self.R[j-6:j])

            # Permutate B[1] to B[8] using the S-Boxes
            j = 0
            Bn = [0] * 32
            pos = 0
            while j < 8:
                # Work out the offsets
                m = (B[j][0] << 1) + B[j][5]
                n = (B[j][1] << 3) + (B[j][2] << 2) + (B[j][3] << 1) + B[j][4]

                # Find the permutation value
                v = des.__sbox[j][(m << 4) + n]

                # Turn value into bits, add it to result: Bn
                Bn[pos] = (v & 8) >> 3
                Bn[pos + 1] = (v & 4) >> 2
                Bn[pos + 2] = (v & 2) >> 1
                Bn[pos + 3] = v & 1

                pos += 4
                j += 1

            # Permutate the concatination of B[1] to B[8] (Bn)
            self.R = self.__permutate(des.__p, Bn)

            # Xor with L[i - 1]
            self.R = list(map(lambda x, y: x ^ y, self.R, self.L))
            # Optimization: This now replaces the below commented code
            #j = 0
            #while j < len(self.R):
            #   self.R[j] = self.R[j] ^ self.L[j]
            #   j += 1

            # L[i] becomes R[i - 1]
            self.L = tempR

            i += 1
            iteration += iteration_adjustment

        # Final permutation of R[16]L[16]
        self.final = self.__permutate(des.__fp, self.R + self.L)
        return self.final


    # Data to be encrypted/decrypted
    def crypt(self, data, crypt_type):
        """Crypt the data in blocks, running it through des_crypt()"""

        # Error check the data
        if not data:
            return b''
        if len(data) % self.block_size != 0:
            if crypt_type == des.DECRYPT: # Decryption must work on 8 byte blocks
                raise ValueError("Invalid data length, data must be a multiple of " + str(self.block_size) + " bytes\n.")
            if not self.getPadding():
                raise ValueError("Invalid data length, data must be a multiple of " + str(self.block_size) + " bytes\n. Try setting the optional padding character")
            else:
                data += (self.block_size - (len(data) % self.block_size)) * self.getPadding()
            # print("Len of data: %f" % (len(data) / self.block_size))

        if self.getMode() == CBC:
            if self.getIV():
                iv = self.__String_to_BitList(self.getIV())
            else:
                raise ValueError("For CBC mode, you must supply the Initial Value (IV) for ciphering")

        # Split the data into blocks, crypting each one seperately
        i = 0
        dict = {}
        result = []
        #cached = 0
        #lines = 0
        while i < len(data):
            # Test code for caching encryption results
            #lines += 1
            #if dict.has_key(data[i:i+8]):
                #print("Cached result for: %s" % data[i:i+8])
            #   cached += 1
            #   result.append(dict[data[i:i+8]])
            #   i += 8
            #   continue

            block = self.__String_to_BitList(data[i:i+8])

            # Xor with IV if using CBC mode
            if self.getMode() == CBC:
                if crypt_type == des.ENCRYPT:
                    block = map(lambda x, y: x ^ y, block, iv)
                    #j = 0
                    #while j < len(block):
                    #   block[j] = block[j] ^ iv[j]
                    #   j += 1

                processed_block = self.__des_crypt(block, crypt_type)

                if crypt_type == des.DECRYPT:
                    processed_block = map(lambda x, y: x ^ y, processed_block, iv)
                    #j = 0
                    #while j < len(processed_block):
                    #   processed_block[j] = processed_block[j] ^ iv[j]
                    #   j += 1
                    iv = block
                else:
                    iv = processed_block
            else:
                processed_block = self.__des_crypt(block, crypt_type)


            # Add the resulting crypted block to our list
            #d = self.__BitList_to_String(processed_block)
            #result.append(d)
            result.append(self.__BitList_to_String(processed_block))
            #dict[data[i:i+8]] = d
            i += 8

        # print("Lines: %d, cached: %d" % (lines, cached))

        # Remove the padding from the last block
        if crypt_type == des.DECRYPT and self.getPadding():
            #print("Removing decrypt pad")
            s = result[-1]
            while s[-1] == self.getPadding():
                s = s[:-1]
            result[-1] = s

        # Return the full crypted string
        return b''.join(result)

    def encrypt(self, data, pad=b''):
        """encrypt(data, [pad]) -> string

        data : String to be encrypted
        pad  : Optional argument for encryption padding. Must only be one byte

        The data must be a multiple of 8 bytes and will be encrypted
        with the already specified key. Data does not have to be a
        multiple of 8 bytes if the padding character is supplied, the
        data will then be padded to a multiple of 8 bytes with this
        pad character.
        """
        self.__padding = pad
        return self.crypt(data, des.ENCRYPT)

    def decrypt(self, data, pad=b''):
        """decrypt(data, [pad]) -> string

        data : String to be encrypted
        pad  : Optional argument for decryption padding. Must only be one byte

        The data must be a multiple of 8 bytes and will be decrypted
        with the already specified key. If the optional padding character
        is supplied, then the un-encypted data will have the padding characters
        removed from the end of the string. This pad removal only occurs on the
        last 8 bytes of the data (last data block).
        """
        self.__padding = pad
        return self.crypt(data, des.DECRYPT)


#############################################################################
#               Triple DES                  #
#############################################################################
class triple_des:
    """Triple DES encryption/decrytpion class

    This algorithm uses the DES-EDE3 (when a 24 byte key is supplied) or
    the DES-EDE2 (when a 16 byte key is supplied) encryption methods.
    Supports ECB (Electronic Code Book) and CBC (Cypher Block Chaining) modes.

    pyDes.des(key, [mode], [IV])

    key  -> The encryption key string, must be either 16 or 24 bytes long
    mode -> Optional argument for encryption type, can be either pyDes.ECB
        (Electronic Code Book), pyDes.CBC (Cypher Block Chaining)
    IV   -> Optional string argument, must be supplied if using CBC mode.
        Must be 8 bytes in length.
    """
    def __init__(self, key, mode=ECB, IV=None):
        self.block_size = 8
        self.setMode(mode)
        self.__padding = b''
        self.__iv = IV
        self.setKey(key)

    def getKey(self):
        """getKey() -> string"""
        return self.__key

    def setKey(self, key):
        """Will set the crypting key for this object. Either 16 or 24 bytes long."""
        self.key_size = 24  # Use DES-EDE3 mode
        if len(key) != self.key_size:
            if len(key) == 16: # Use DES-EDE2 mode
                self.key_size = 16
            else:
                raise ValueError("Invalid triple DES key size. Key must be either 16 or 24 bytes long")
        if self.getMode() == CBC:
            if not self.getIV():
                # Use the first 8 bytes of the key
                self.setIV(key[:self.block_size])
            if len(self.getIV()) != self.block_size:
                raise ValueError("Invalid IV, must be 8 bytes in length")
        self.__key1 = des(key[:8], self.getMode(), self.getIV())
        self.__key2 = des(key[8:16], self.getMode(), self.getIV())
        if self.key_size == 16:
            self.__key3 = self.__key1
        else:
            self.__key3 = des(key[16:], self.getMode(), self.getIV())
        self.__key = key

    def getMode(self):
        """getMode() -> pyDes.ECB or pyDes.CBC"""
        return self.__mode

    def setMode(self, mode):
        """Sets the type of crypting mode, pyDes.ECB or pyDes.CBC"""
        self.__mode = mode

    def getIV(self):
        """getIV() -> string"""
        return self.__iv

    def setIV(self, IV):
        """Will set the Initial Value, used in conjunction with CBC mode"""
        self.__iv = IV

    def encrypt(self, data, pad=b''):
        """encrypt(data, [pad]) -> string

        data : String to be encrypted
        pad  : Optional argument for encryption padding. Must only be one byte

        The data must be a multiple of 8 bytes and will be encrypted
        with the already specified key. Data does not have to be a
        multiple of 8 bytes if the padding character is supplied, the
        data will then be padded to a multiple of 8 bytes with this
        pad character.
        """
        if self.getMode() == CBC:
            self.__key1.setIV(self.getIV())
            self.__key2.setIV(self.getIV())
            self.__key3.setIV(self.getIV())
            i = 0
            result = []
            while i < len(data):
                block = self.__key1.encrypt(data[i:i+8], pad)
                block = self.__key2.decrypt(block)
                block = self.__key3.encrypt(block)
                self.__key1.setIV(block)
                self.__key2.setIV(block)
                self.__key3.setIV(block)
                result.append(block)
                i += 8
            return b''.join(result)
        else:
            data = self.__key1.encrypt(data, pad)
            data = self.__key2.decrypt(data)
            return self.__key3.encrypt(data)

    def decrypt(self, data, pad=b''):
        """decrypt(data, [pad]) -> string

        data : String to be encrypted
        pad  : Optional argument for decryption padding. Must only be one byte

        The data must be a multiple of 8 bytes and will be decrypted
        with the already specified key. If the optional padding character
        is supplied, then the un-encypted data will have the padding characters
        removed from the end of the string. This pad removal only occurs on the
        last 8 bytes of the data (last data block).
        """
        if self.getMode() == CBC:
            self.__key1.setIV(self.getIV())
            self.__key2.setIV(self.getIV())
            self.__key3.setIV(self.getIV())
            i = 0
            result = []
            while i < len(data):
                iv = data[i:i+8]
                block = self.__key3.decrypt(iv)
                block = self.__key2.encrypt(block)
                block = self.__key1.decrypt(block, pad)
                self.__key1.setIV(iv)
                self.__key2.setIV(iv)
                self.__key3.setIV(iv)
                result.append(block)
                i += 8
            return b''.join(result)
        else:
            data = self.__key3.decrypt(data)
            data = self.__key2.encrypt(data)
            return self.__key1.decrypt(data, pad)


#############################################################################
#               Examples                    #
#############################################################################
def example_triple_des():
    from time import time

    # Utility module
    from binascii import unhexlify as unhex

    # example shows triple-des encryption using the des class
    print("Example of triple DES encryption in default ECB mode (DES-EDE3)\n")

    print("Triple des using the des class (3 times)")
    t = time()
    k1 = des(unhex("133457799BBCDFF1"))
    k2 = des(unhex("1122334455667788"))
    k3 = des(unhex("77661100DD223311"))
    d = "Triple DES test string, to be encrypted and decrypted..."
    print("Key1:      %s" % k1.getKey())
    print("Key2:      %s" % k2.getKey())
    print("Key3:      %s" % k3.getKey())
    print("Data:      %s" % d)

    e1 = k1.encrypt(d)
    e2 = k2.decrypt(e1)
    e3 = k3.encrypt(e2)
    print("Encrypted: " + e3)

    d3 = k3.decrypt(e3)
    d2 = k2.encrypt(d3)
    d1 = k1.decrypt(d2)
    print("Decrypted: " + d1)
    print("DES time taken: %f (%d crypt operations)" % (time() - t, 6 * (len(d) / 8)))
    print("")

    # Example below uses the triple-des class to achieve the same as above
    print("Now using triple des class")
    t = time()
    t1 = triple_des(unhex("133457799BBCDFF1112233445566778877661100DD223311"))
    print("Key:       %s" % t1.getKey())
    print("Data:      %s" % d)

    td1 = t1.encrypt(d)
    print("Encrypted: " + td1)

    td2 = t1.decrypt(td1)
    print("Decrypted: " + td2)

    print("Triple DES time taken: %f (%d crypt operations)" % (time() - t, 6 * (len(d) / 8)))

def example_des():
    from time import time

    # example of DES encrypting in CBC mode with the IV of "\0\0\0\0\0\0\0\0"
    print("Example of DES encryption using CBC mode\n")
    t = time()
    k = des("DESCRYPT", CBC, "\0\0\0\0\0\0\0\0")
    data = "DES encryption algorithm"
    print("Key      : " + k.getKey())
    print("Data     : " + data)

    d = k.encrypt(data)
    print("Encrypted: " + d)

    d = k.decrypt(d)
    print("Decrypted: " + d)
    print("DES time taken: %f (6 crypt operations)" % (time() - t))
    print("")

def __test__():
    example_des()
    example_triple_des()


def __fulltest__():
    # This should not produce any unexpected errors or exceptions
    from binascii import unhexlify as unhex
    from binascii import hexlify as dohex

    __test__()
    print("")

    k = des("\0\0\0\0\0\0\0\0", CBC, "\0\0\0\0\0\0\0\0")
    d = k.encrypt("DES encryption algorithm")
    if k.decrypt(d) != "DES encryption algorithm":
        print("Test 1 Error: Unencypted data block does not match start data")

    k = des("\0\0\0\0\0\0\0\0", CBC, "\0\0\0\0\0\0\0\0")
    d = k.encrypt("Default string of text", '*')
    if k.decrypt(d, "*") != "Default string of text":
        print("Test 2 Error: Unencypted data block does not match start data")

    k = des("\r\n\tABC\r\n")
    d = k.encrypt("String to Pad", '*')
    if k.decrypt(d) != "String to Pad***":
        print("'%s'" % k.decrypt(d))
        print("Test 3 Error: Unencypted data block does not match start data")

    k = des("\r\n\tABC\r\n")
    d = k.encrypt(unhex("000102030405060708FF8FDCB04080"), unhex("44"))
    if k.decrypt(d, unhex("44")) != unhex("000102030405060708FF8FDCB04080"):
        print("Test 4a Error: Unencypted data block does not match start data")
    if k.decrypt(d) != unhex("000102030405060708FF8FDCB0408044"):
        print("Test 4b Error: Unencypted data block does not match start data")

    k = triple_des("MyDesKey\r\n\tABC\r\n0987*543")
    d = k.encrypt(unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080"))
    if k.decrypt(d) != unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080"):
        print("Test 5 Error: Unencypted data block does not match start data")

    k = triple_des("\r\n\tABC\r\n0987*543")
    d = k.encrypt(unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080"))
    if k.decrypt(d) != unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080"):
        print("Test 6 Error: Unencypted data block does not match start data")

    k = triple_des("MyDesKey\r\n\tABC\r\n0987*54B", CBC, "12341234")
    d = k.encrypt(unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080"))
    if k.decrypt(d) != unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080"):
        print("Test 7 Error: Triple DES CBC failed.")

    k = triple_des("MyDesKey\r\n\tABC\r\n0987*54B", CBC, "12341234")
    d = k.encrypt(unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDC"), '.')
    if k.decrypt(d, '.') != unhex("000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDCB04080000102030405060708FF8FDC"):
        print("Test 8 Error: Triple DES CBC with padding failed.")

def __filetest__():
    from time import time

    f = open("pyDes.py", "rb+")
    d = f.read()
    f.close()

    t = time()
    k = des("MyDESKey")

    d = k.encrypt(d, " ")
    f = open("pyDes.py.enc", "wb+")
    f.write(d)
    f.close()

    d = k.decrypt(d, " ")
    f = open("pyDes.py.dec", "wb+")
    f.write(d)
    f.close()
    print("DES file test time: %f" % (time() - t))

def __profile__():
    import profile
    profile.run('__fulltest__()')
    #profile.run('__filetest__()')

if __name__ == '__main__':
    __test__()
    #__fulltest__()
    #__filetest__()
    #__profile__()



# -*- coding: utf-8 -*-
#
#  SelfTest/Hash/SHA.py: Self-test for the SHA-1 hash function
#
# =======================================================================
# Copyright (C) 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# =======================================================================
#

"""Self-test suite for CryptoPlus.Hash.SHA"""

__revision__ = "$Id$"

# Test vectors from various sources
# This is a list of (expected_result, input[, description]) tuples.
test_data = [
    # FIPS PUB 180-2, A.1 - "One-Block Message"
    ('a9993e364706816aba3e25717850c26c9cd0d89d', 'abc'),

    # FIPS PUB 180-2, A.2 - "Multi-Block Message"
    ('84983e441c3bd26ebaae4aa1f95129e5e54670f1',
        'abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq'),

    # FIPS PUB 180-2, A.3 - "Long Message"
#    ('34aa973cd4c4daa4f61eeb2bdbad27316534016f',
#        'a' * 10**6,
#         '"a" * 10**6'),

    # RFC 3174: Section 7.3, "TEST4" (multiple of 512 bits)
    ('dea356a2cddd90c7a7ecedc5ebb563934f460452',
        "01234567" * 80,
        '"01234567" * 80'),
]

def get_tests():
    from CryptoPlus.Hash import SHA
    from common import make_hash_tests
    return make_hash_tests(SHA, "SHA", test_data)

if __name__ == '__main__':
    import unittest
    suite = lambda: unittest.TestSuite(get_tests())
    unittest.main(defaultTest='suite')

# vim:set ts=4 sw=4 sts=4 expandtab:



from pkg_resources import parse_version
import Crypto

if parse_version(Crypto.__version__) > parse_version("2.0.1"):
        del Crypto
        try:
               from Crypto.Util.python_compat import *
        except:
               try:
                       from Crypto.Util.py21compat import *
               except:
                       from Crypto.Util.py3compat import *



# -*- coding: utf-8 -*-
#
#  SelfTest/Hash/common.py: Common code for CryptoPlus.SelfTest.Hash
#
# =======================================================================
# Copyright (C) 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# =======================================================================
#

"""Self-testing for PyCryptoPlus hash modules"""

__revision__ = "$Id$"

import sys
import unittest
import binascii
import string

# For compatibility with Python 2.1 and Python 2.2
if sys.hexversion < 0x02030000:
    # Python 2.1 doesn't have a dict() function
    # Python 2.2 dict() function raises TypeError if you do dict(MD5='blah')
    def dict(**kwargs):
        return kwargs.copy()
else:
    dict = __builtins__['dict']


class HashSelfTest(unittest.TestCase):

    def __init__(self, hashmod, description, expected, input):
        unittest.TestCase.__init__(self)
        self.hashmod = hashmod
        self.expected = expected
        self.input = input
        self.description = description

    def shortDescription(self):
        return self.description

    def runTest(self):
        h = self.hashmod.new()
        h.update(self.input)

        out1 = binascii.b2a_hex(h.digest())
        out2 = h.hexdigest()

        h = self.hashmod.new(self.input)

        out3 = h.hexdigest()
        out4 = binascii.b2a_hex(h.digest())

        self.assertEqual(self.expected, out1)
        self.assertEqual(self.expected, out2)
        self.assertEqual(self.expected, out3)
        self.assertEqual(self.expected, out4)

class MACSelfTest(unittest.TestCase):

    def __init__(self, hashmod, description, expected_dict, input, key, hashmods):
        unittest.TestCase.__init__(self)
        self.hashmod = hashmod
        self.expected_dict = expected_dict
        self.input = input
        self.key = key
        self.hashmods = hashmods
        self.description = description

    def shortDescription(self):
        return self.description

    def runTest(self):
        for hashname in self.expected_dict.keys():
            hashmod = self.hashmods[hashname]
            key = binascii.a2b_hex(self.key)
            data = binascii.a2b_hex(self.input)

            # Strip whitespace from the expected string (which should be in lowercase-hex)
            expected = self.expected_dict[hashname]
            for ch in string.whitespace:
                expected = expected.replace(ch, "")

            h = self.hashmod.new(key, digestmod=hashmod)
            h.update(data)
            out1 = binascii.b2a_hex(h.digest())
            out2 = h.hexdigest()

            h = self.hashmod.new(key, data, hashmod)

            out3 = h.hexdigest()
            out4 = binascii.b2a_hex(h.digest())

            # Test .copy()
            h2 = h.copy()
            h.update("blah blah blah")  # Corrupt the original hash object
            out5 = binascii.b2a_hex(h2.digest())    # The copied hash object should return the correct result

            self.assertEqual(expected, out1)
            self.assertEqual(expected, out2)
            self.assertEqual(expected, out3)
            self.assertEqual(expected, out4)
            self.assertEqual(expected, out5)

def make_hash_tests(module, module_name, test_data):
    tests = []
    for i in range(len(test_data)):
        row = test_data[i]
        if len(row) < 3:
            (expected, input) = row
            description = repr(input)
        else:
            (expected, input, description) = row
        name = "%s #%d: %s" % (module_name, i+1, description)
        tests.append(HashSelfTest(module, name, expected, input))
    return tests

def make_mac_tests(module, module_name, test_data, hashmods):
    tests = []
    for i in range(len(test_data)):
        row = test_data[i]
        (key, data, results, description) = row
        name = "%s #%d: %s" % (module_name, i+1, description)
        tests.append(MACSelfTest(module, name, results, data, key, hashmods))
    return tests

# vim:set ts=4 sw=4 sts=4 expandtab:



"""Imports Crypto PublicKey

Now you can do:
    >>> from CryptoPlus.PublicKey import *
OR:
    >>> from CryptoPlus.PublicKey import RSA
but not:
    >>> import CryptoPlus.PublicKey.RSA
"""

from Crypto.PublicKey import *



from .blockcipher import *
from .rijndael import rijndael

def new(key,mode=MODE_ECB,IV=None,counter=None,segment_size=None,blocksize=None):
    """Create a new cipher object

    Wrapper for pure python implementation rijndael.py

        key = raw string containing the key
            -> supported key size are 16, 24 and 32 bytes
        mode = python_Rijndael.MODE_ECB/CBC/CFB/OFB/CTR/XTS/CMAC, default is ECB
            -> for every mode, except ECB and CTR, it is important to construct a seperate cipher for encryption and decryption
        IV = IV as a raw string, default is "all zero" IV
            -> needed for CBC, CFB and OFB mode
        counter = counter object (CryptoPlus.Util.util.Counter)
            -> only needed for CTR mode
            -> use a seperate counter object for the cipher and decipher: the counter is updated directly, not a copy
                see CTR example further on in the docstring
        segment_size = amount of bits to use from the keystream in each chain part
            -> supported values: multiple of 8 between 8 and the blocksize
               of the cipher (only per byte access possible), default is 8
            -> only needed for CFB mode
        blocksize = blocksize in bytes
            -> supported blocksizes are 16, 24 and 32 bytes, must be 16 if XTS mode.

    EXAMPLES:
    **********
    IMPORTING:
    -----------
    >>> import codecs
    >>> from CryptoPlus.Cipher import python_Rijndael

    EXAMPLE:
    --------
    24 byte block, 32 byte key (http://fp.gladman.plus.com/cryptography_technology/rijndael/)
    >>> key = codecs.decode('2b7e151628aed2a6abf7158809cf4f3c762e7160f38b4da56a784d9045190cfe', 'hex')
    >>> plaintext =codecs.decode('3243f6a8885a308d313198a2e03707344a4093822299f31d', 'hex')
    >>> cipher = python_Rijndael.new(key,python_Rijndael.MODE_ECB,blocksize=24)
    >>> codecs.encode(cipher.encrypt(plaintext), 'hex')
    b'0ebacf199e3315c2e34b24fcc7c46ef4388aa475d66c194c'

    CBC EXAMPLE (plaintext = 3 blocksizes) (AES):
    -----------------------------------------
    >>> key = codecs.decode('2b7e151628aed2a6abf7158809cf4f3c', 'hex')
    >>> IV = codecs.decode('000102030405060708090a0b0c0d0e0f', 'hex')
    >>> plaintext1 = codecs.decode('6bc1bee22e409f96e93d7e117393172a', 'hex')
    >>> plaintext2 = codecs.decode('ae2d8a571e03ac9c9eb76fac45af8e51', 'hex')
    >>> plaintext3 = codecs.decode('30c81c46a35ce411e5fbc1191a0a52ef', 'hex')
    >>> cipher = python_Rijndael.new(key,python_Rijndael.MODE_CBC,IV,blocksize=16)
    >>> ciphertext = cipher.encrypt(plaintext1 + plaintext2 + plaintext3)
    >>> codecs.encode(ciphertext, 'hex')
    b'7649abac8119b246cee98e9b12e9197d5086cb9b507219ee95db113a917678b273bed6b8e3c1743b7116e69e22229516'
    >>> decipher = python_Rijndael.new(key,python_Rijndael.MODE_CBC,IV,blocksize=16)
    >>> plaintext = decipher.decrypt(ciphertext)
    >>> codecs.encode(plaintext, 'hex')
    b'6bc1bee22e409f96e93d7e117393172aae2d8a571e03ac9c9eb76fac45af8e5130c81c46a35ce411e5fbc1191a0a52ef'

    XTS EXAMPLE:
    ------------
    (Examples for XTS-AES)
    XTS-AES-128 applied for a data unit of 512 bytes
    testvector: http://grouper.ieee.org/groups/1619/email/pdf00086.pdf

    >>> key = (codecs.decode('27182818284590452353602874713526', 'hex'),codecs.decode('31415926535897932384626433832795', 'hex'))
    >>> plaintext = codecs.decode('000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f606162636465666768696a6b6c6d6e6f707172737475767778797a7b7c7d7e7f808182838485868788898a8b8c8d8e8f909192939495969798999a9b9c9d9e9fa0a1a2a3a4a5a6a7a8a9aaabacadaeafb0b1b2b3b4b5b6b7b8b9babbbcbdbebfc0c1c2c3c4c5c6c7c8c9cacbcccdcecfd0d1d2d3d4d5d6d7d8d9dadbdcdddedfe0e1e2e3e4e5e6e7e8e9eaebecedeeeff0f1f2f3f4f5f6f7f8f9fafbfcfdfeff000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f606162636465666768696a6b6c6d6e6f707172737475767778797a7b7c7d7e7f808182838485868788898a8b8c8d8e8f909192939495969798999a9b9c9d9e9fa0a1a2a3a4a5a6a7a8a9aaabacadaeafb0b1b2b3b4b5b6b7b8b9babbbcbdbebfc0c1c2c3c4c5c6c7c8c9cacbcccdcecfd0d1d2d3d4d5d6d7d8d9dadbdcdddedfe0e1e2e3e4e5e6e7e8e9eaebecedeeeff0f1f2f3f4f5f6f7f8f9fafbfcfdfeff', 'hex')
    >>> cipher = python_Rijndael.new(key,python_Rijndael.MODE_XTS,blocksize=16)
    >>> ciphertext = cipher.encrypt(plaintext)
    >>> codecs.encode(ciphertext, 'hex')
    b'27a7479befa1d476489f308cd4cfa6e2a96e4bbe3208ff25287dd3819616e89cc78cf7f5e543445f8333d8fa7f56000005279fa5d8b5e4ad40e736ddb4d35412328063fd2aab53e5ea1e0a9f332500a5df9487d07a5c92cc512c8866c7e860ce93fdf166a24912b422976146ae20ce846bb7dc9ba94a767aaef20c0d61ad02655ea92dc4c4e41a8952c651d33174be51a10c421110e6d81588ede82103a252d8a750e8768defffed9122810aaeb99f9172af82b604dc4b8e51bcb08235a6f4341332e4ca60482a4ba1a03b3e65008fc5da76b70bf1690db4eae29c5f1badd03c5ccf2a55d705ddcd86d449511ceb7ec30bf12b1fa35b913f9f747a8afd1b130e94bff94effd01a91735ca1726acd0b197c4e5b03393697e126826fb6bbde8ecc1e08298516e2c9ed03ff3c1b7860f6de76d4cecd94c8119855ef5297ca67e9f3e7ff72b1e99785ca0a7e7720c5b36dc6d72cac9574c8cbbc2f801e23e56fd344b07f22154beba0f08ce8891e643ed995c94d9a69c9f1b5f499027a78572aeebd74d20cc39881c213ee770b1010e4bea718846977ae119f7a023ab58cca0ad752afe656bb3c17256a9f6e9bf19fdd5a38fc82bbe872c5539edb609ef4f79c203ebb140f2e583cb2ad15b4aa5b655016a8449277dbd477ef2c8d6c017db738b18deb4a427d1923ce3ff262735779a418f20a282df920147beabe421ee5319d0568'
    >>> decipher = python_Rijndael.new(key,python_Rijndael.MODE_XTS,blocksize=16)
    >>> codecs.encode(decipher.decrypt(ciphertext), 'hex')
    b'000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f606162636465666768696a6b6c6d6e6f707172737475767778797a7b7c7d7e7f808182838485868788898a8b8c8d8e8f909192939495969798999a9b9c9d9e9fa0a1a2a3a4a5a6a7a8a9aaabacadaeafb0b1b2b3b4b5b6b7b8b9babbbcbdbebfc0c1c2c3c4c5c6c7c8c9cacbcccdcecfd0d1d2d3d4d5d6d7d8d9dadbdcdddedfe0e1e2e3e4e5e6e7e8e9eaebecedeeeff0f1f2f3f4f5f6f7f8f9fafbfcfdfeff000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f606162636465666768696a6b6c6d6e6f707172737475767778797a7b7c7d7e7f808182838485868788898a8b8c8d8e8f909192939495969798999a9b9c9d9e9fa0a1a2a3a4a5a6a7a8a9aaabacadaeafb0b1b2b3b4b5b6b7b8b9babbbcbdbebfc0c1c2c3c4c5c6c7c8c9cacbcccdcecfd0d1d2d3d4d5d6d7d8d9dadbdcdddedfe0e1e2e3e4e5e6e7e8e9eaebecedeeeff0f1f2f3f4f5f6f7f8f9fafbfcfdfeff'

    using data sequence number n

    >>> key = (codecs.decode('fffefdfcfbfaf9f8f7f6f5f4f3f2f1f0', 'hex'),codecs.decode('22222222222222222222222222222222', 'hex'))
    >>> plain =codecs.decode('4444444444444444444444444444444444444444444444444444444444444444', 'hex')
    >>> n = codecs.decode('3333333333', 'hex')
    >>> cipher = python_Rijndael.new(key,python_Rijndael.MODE_XTS,blocksize=16)
    >>> ciphertext = cipher.encrypt(plain,n)
    >>> codecs.encode(ciphertext, 'hex')
    b'af85336b597afc1a900b2eb21ec949d292df4c047e0b21532186a5971a227a89'
    >>> decipher = python_Rijndael.new(key,python_Rijndael.MODE_XTS,blocksize=16)
    >>> codecs.encode(decipher.decrypt(ciphertext,n), 'hex')
    b'4444444444444444444444444444444444444444444444444444444444444444'
    """
    return python_Rijndael(key,mode,IV,counter,blocksize,segment_size)

class python_Rijndael(BlockCipher):
    key_error_message = ("Key should be 128, 192 or 256 bits")

    def __init__(self,key,mode,IV,counter,blocksize,segment_size):
        if blocksize not in (16,24,32):
                raise ValueError("Blocksize should be 16, 24 or 32")
        cipher_module = rijndael
        args = {'block_size':blocksize}
        self.blocksize = blocksize
        BlockCipher.__init__(self,key,mode,IV,counter,cipher_module,segment_size,args)

    def keylen_valid(self,key):
        return len(key) in (16,24,32)

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from . import pypbkdf2
from CryptoPlus.Hash import SHA as SHA1, HMAC

__all__ = ['new']

def new(passphrase, salt, iterations=1000, digestmodule=SHA1, macmodule=HMAC):
        """PKCS#5 v2.0 Password-Based Key Derivation
        
        passphrase      = the passphrase, supplied as a raw string, to make a key from
        salt            = salt as raw string
        iterations      = amount of iterations (default = 1000)
        digestmodule    = digest function to use, supply as module
                           example: python_SHA from CryptoPlus.Hash
                           default: SHA1
        macmodule       = mac function to use, supply as module
                           example: HMAC from CryptoPlus.Hash
                           default: HMAC

                => macmodule & digestmodule construct the pseudorandom function
                        => by default: HMAC-SHA1

        Examples: (from: http://www.ietf.org/rfc/rfc3962.txt)
        ==========

        >>> from CryptoPlus.Hash import python_PBKDF2

        >>> passphrase = b"password"
        >>> salt = b"ATHENA.MIT.EDUraeburn"
        >>> iterations = 1
        >>> hasher = python_PBKDF2.new(passphrase,salt,iterations)
        >>> hasher.hexread(16)
        'cdedb5281bb2f801565a1122b2563515'

        >>> passphrase = b"password"
        >>> salt = b"ATHENA.MIT.EDUraeburn"
        >>> iterations = 1200
        >>> hasher = python_PBKDF2.new(passphrase,salt,iterations)
        >>> hasher.hexread(32)
        '5c08eb61fdf71e4e4ec3cf6ba1f5512ba7e52ddbc5e5142f708a31e2e62b1e13'

        >>> passphrase = b"X"*64
        >>> salt = b"pass phrase equals block size"
        >>> iterations = 1200
        >>> hasher = python_PBKDF2.new(passphrase,salt,iterations)
        >>> hasher.hexread(32)
        '139c30c0966bc32ba55fdbf212530ac9c5ec59f1a452f5cc9ad940fea0598ed1'

        >>> passphrase = b"X"*65
        >>> salt = b"pass phrase exceeds block size"
        >>> iterations = 1200
        >>> hasher = python_PBKDF2.new(passphrase,salt,iterations)
        >>> hasher.hexread(32)
        '9ccad6d468770cd51b10e6a68721be611a8b4d282601db3b36be9246915ec82a'
        """
        return pypbkdf2.PBKDF2(passphrase, salt, iterations, digestmodule, macmodule)



# -*- coding: utf-8 -*-
#
#  SelfTest/Hash/common.py: Common code for CryptoPlus.SelfTest.Hash
#
# =======================================================================
# Copyright (C) 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# =======================================================================
#

"""Self-testing for PyCryptoPlus hash modules"""

__revision__ = "$Id$"

import sys
import unittest
from binascii import a2b_hex, b2a_hex

# For compatibility with Python 2.1 and Python 2.2
if sys.hexversion < 0x02030000:
    # Python 2.1 doesn't have a dict() function
    # Python 2.2 dict() function raises TypeError if you do dict(MD5='blah')
    def dict(**kwargs):
        return kwargs.copy()
else:
    dict = __builtins__['dict']

class _NoDefault: pass        # sentinel object
def _extract(d, k, default=_NoDefault):
    """Get an item from a dictionary, and remove it from the dictionary."""
    try:
        retval = d[k]
    except KeyError:
        if default is _NoDefault:
            raise
        return default
    del d[k]
    return retval

# Generic cipher test case
class CipherSelfTest(unittest.TestCase):

    def __init__(self, module, params):
        unittest.TestCase.__init__(self)
        self.module = module

        # Extract the parameters
        params = params.copy()
        self.description = _extract(params, 'description')
        self.key = _extract(params, 'key')
        self.plaintext = _extract(params, 'plaintext')
        self.ciphertext = _extract(params, 'ciphertext')

        mode = _extract(params, 'mode', None)
        if mode is not None:
            # Block cipher
            self.mode = getattr(self.module, "MODE_" + mode)
            self.iv = _extract(params, 'iv', None)
        else:
            # Stream cipher
            self.mode = None
            self.iv = None

        self.extra_params = params

    def shortDescription(self):
        return self.description

    def _new(self):
        if self.mode is None:
            # Stream cipher
            return self.module.new(a2b_hex(self.key), **self.extra_params)
        elif self.iv is None:
            # Block cipher without iv
            return self.module.new(a2b_hex(self.key), self.mode, **self.extra_params)
        else:
            # Block cipher with iv
            return self.module.new(a2b_hex(self.key), self.mode, a2b_hex(self.iv), **self.extra_params)

    def runTest(self):
        plaintext = a2b_hex(self.plaintext)
        ciphertext = a2b_hex(self.ciphertext)

        ct1 = b2a_hex(self._new().encrypt(plaintext))
        pt1 = b2a_hex(self._new().decrypt(ciphertext))
        ct2 = b2a_hex(self._new().encrypt(plaintext))
        pt2 = b2a_hex(self._new().decrypt(ciphertext))

        self.assertEqual(self.ciphertext, ct1)
        self.assertEqual(self.ciphertext, ct2)
        self.assertEqual(self.plaintext, pt1)
        self.assertEqual(self.plaintext, pt2)

def make_block_tests(module, module_name, test_data):
    tests = []
    for i in range(len(test_data)):
        row = test_data[i]

        # Build the "params" dictionary
        params = {'mode': 'ECB'}
        if len(row) == 3:
            (params['plaintext'], params['ciphertext'], params['key']) = row
        elif len(row) == 4:
            (params['plaintext'], params['ciphertext'], params['key'], params['description']) = row
        elif len(row) == 5:
            (params['plaintext'], params['ciphertext'], params['key'], params['description'], extra_params) = row
            params.update(extra_params)
        else:
            raise AssertionError("Unsupported tuple size %d" % (len(row),))

        # Build the display-name for the test
        p2 = params.copy()
        p_key = _extract(p2, 'key')
        p_plaintext = _extract(p2, 'plaintext')
        p_ciphertext = _extract(p2, 'ciphertext')
        p_description = _extract(p2, 'description', None)
        p_mode = p2.get('mode', 'ECB')
        if p_mode == 'ECB':
            _extract(p2, 'mode', 'ECB')

        if p_description is not None:
            description = p_description
        elif p_mode == 'ECB' and not p2:
            description = "p=%s, k=%s" % (p_plaintext, p_key)
        else:
            description = "p=%s, k=%s, %r" % (p_plaintext, p_key, p2)
        name = "%s #%d: %s" % (module_name, i+1, description)
        params['description'] = name

        # Add the test to the test suite
        tests.append(CipherSelfTest(module, params))
    return tests

def make_stream_tests(module, module_name, test_data):
    tests = []
    for i in range(len(test_data)):
        row = test_data[i]

        # Build the "params" dictionary
        params = {}
        if len(row) == 3:
            (params['plaintext'], params['ciphertext'], params['key']) = row
        elif len(row) == 4:
            (params['plaintext'], params['ciphertext'], params['key'], params['description']) = row
        elif len(row) == 5:
            (params['plaintext'], params['ciphertext'], params['key'], params['description'], extra_params) = row
            params.update(extra_params)
        else:
            raise AssertionError("Unsupported tuple size %d" % (len(row),))

        # Build the display-name for the test
        p2 = params.copy()
        p_key = _extract(p2, 'key')
        p_plaintext = _extract(p2, 'plaintext')
        p_ciphertext = _extract(p2, 'ciphertext')
        p_description = _extract(p2, 'description', None)

        if p_description is not None:
            description = p_description
        elif not p2:
            description = "p=%s, k=%s" % (p_plaintext, p_key)
        else:
            description = "p=%s, k=%s, %r" % (p_plaintext, p_key, p2)
        name = "%s #%d: %s" % (module_name, i+1, description)
        params['description'] = name

        # Add the test to the test suite
        tests.append(CipherSelfTest(module, params))
    return tests

# vim:set ts=4 sw=4 sts=4 expandtab:



from Crypto.Cipher import ARC4, XOR

__all__ = ["AES","python_AES","python_DES","python_DES3","DES","DES3","Blowfish","python_Blowfish","python_Twofish","python_Serpent","python_Rijndael","ARC4","ARC2","CAST","XOR","python_PRESENT"]

try:
        import Crypto.Cipher.IDEA
        __all__.append("IDEA")
        __all__.append("RC5")
except ImportError:
        pass



"""Imports Crypto Protocol

Now you can do:
    >>> from CryptoPlus.Protocol import *
OR:
    >>> from CryptoPlus.Protocol import XXX
but not:
    >>> import CryptoPlus.Protocol.XXX
"""
from Crypto.Protocol import *



from .pysha384 import sha384

__all__ = ['new','digest_size']

def new(data=None):
        """Create a new pure python SHA-384 hash object
        
        data =  initial input (raw string) to the hashing object
                if present, the method call update(arg) is made
        
        EXAMPLE: FIPS 180-2
        =========
        
        >>> from CryptoPlus.Hash import python_SHA384
        
        >>> message = b"abc"
        >>> hasher = python_SHA384.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        'cb00753f45a35e8bb5a03d699ac65007272c32ab0eded1631a8b605a43ff5bed8086072ba1e7cc2358baeca134c825a7'

        >>> message = b"abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu"
        >>> hasher = python_SHA384.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        '09330c33f71147e83d192fc782cd1b4753111b173b3b05d22fa08086e3b0f712fcc7c71a557e2db966c3e9fa91746039'
        """
        return sha384(data)
        
digest_size = sha384.digest_size



from .pyradiogatun import RadioGatunType

__all__ = ['new']

def new(data=None,wl=64):
    """Create a new pure python RadioGatun hash object

    wl   = wordlength (in bits) of the RadioGatun hash method
              between 1 and 64 (default = 64)
    data =  if present, the method call update(arg) is made

    EXAMPLES: (testvectors from: http://radiogatun.noekeon.org/)
    ==========
    >>> from CryptoPlus.Hash import python_RadioGatun
    
    radiogatun[64]
    ---------------
    >>> hasher = python_RadioGatun.new()
    >>> hasher.update(b'1234567890123456')
    >>> hasher.hexdigest()
    'caaec14b5b4a7960d6854709770e3071d635d60224f58aa385867e549ef4cc42'

    >>> hasher = python_RadioGatun.new()
    >>> hasher.update(b'Santa Barbara, California')
    >>> hasher.hexdigest()
    '0d08daf2354fa95aaa5b6a50f514384ecdd35940252e0631002e600e13cd285f'
    
    radiogatun[32]
    ---------------
    >>> hasher = python_RadioGatun.new(wl=32)
    >>> hasher.update(b'1234567890123456')
    >>> hasher.hexdigest()
    '59612324f3f42d3096e69125d2733b86143ae668ae9ed561ad785e0eac8dba25'

    >>> hasher = python_RadioGatun.new(wl=32)
    >>> hasher.update(b'Santa Barbara, California')
    >>> hasher.hexdigest()
    '041666388ef9655d48996a66dada1193d6646012a7b25a24fb10e6075cf0fc54'
    """

    crypto = RadioGatunType(wl)
    if data:
        crypto.update(data)

    return crypto

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    print("DOCTEST running... no messages = all good")
    _test()



# =============================================================================
# Copyright (c) 2008 Christophe Oosterlynck <christophe.oosterlynck_AT_gmail.com>
#                    & NXP ( Philippe Teuwen <philippe.teuwen_AT_nxp.com> )
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# =============================================================================

"""Module for padding functions

padding info here: http://en.wikipedia.org/wiki/Padding_(cryptography)
"""
    
import random

PAD = 0
UNPAD = 1

def bitPadding (padData, direction, length=None):
        """Pad a string using bitPadding

            padData = raw string to pad/unpad
            direction = PAD or UNPAD
            length = amount of bytes the padded string should be a multiple of
                     (length variable is not used when unpadding)
            
            returns: (un)padded raw string
            
            A new block full of padding will be added when padding data that is
            already a multiple of the length.
            
            Example:
            =========
            >>> from CryptoPlus.Util import padding

            >>> padding.bitPadding(b'test', padding.PAD, 8)
            b'test\\x80\\x00\\x00\\x00'
            >>> padding.bitPadding(_,padding.UNPAD)
            b'test'"""
        if direction == PAD:
            if length == None:
                raise ValueError("Supply a valid length")
            return __bitPadding(padData, length)
        elif direction == UNPAD:
            return __bitPadding_unpad(padData)
        else:
            raise ValueError("Supply a valid direction")

def __bitPadding (toPad,length):
    padded = toPad + b'\x80' + b'\x00'*(length - len(toPad)%length -1)
    return padded

def __bitPadding_unpad (padded):
    if padded.rstrip(b'\x00')[-1:] == b'\x80':
        return padded.rstrip(b'\x00')[:-1]
    else:
        return padded

def zerosPadding (padData, direction, length=None):
        """Pad a string using zerosPadding

            padData = raw string to pad/unpad
            direction = PAD or UNPAD
                        beware: padding and unpadding a string ending in 0's
                                will remove those 0's too
            length = amount of bytes the padded string should be a multiple of
                     (length variable is not used when unpadding)
            
            returns: (un)padded raw string
            
            No padding will be added when padding data that is already a
            multiple of the given length.
            
            Example:
            =========
            >>> from CryptoPlus.Util import padding

            >>> padding.zerosPadding(b'12345678',padding.PAD,16)
            b'12345678\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x00'
            >>> padding.zerosPadding(_,padding.UNPAD)
            b'12345678'"""
        if direction == PAD:
            if length == None:
                raise ValueError("Supply a valid length")
            return __zerosPadding(padData, length)
        elif direction == UNPAD:
            return __zerosPadding_unpad(padData)
        else:
            raise ValueError("Supply a valid direction")

def __zerosPadding (toPad, length):
    padLength = (length - len(toPad))%length
    return toPad + b'\x00'*padLength

def __zerosPadding_unpad (padded ):
    return padded.rstrip(b'\x00')

def PKCS7(padData, direction, length=None):
        """Pad a string using PKCS7

            padData = raw string to pad/unpad
            direction = PAD or UNPAD
            length = amount of bytes the padded string should be a multiple of
                     (length variable is not used when unpadding)
            
            returns: (un)padded raw string
            
            A new block full of padding will be added when padding data that is
            already a multiple of the given length.
            
            Example:
            =========
            >>> from CryptoPlus.Util import padding

            >>> padding.PKCS7(b'12345678',padding.PAD,16)
            b'12345678\\x08\\x08\\x08\\x08\\x08\\x08\\x08\\x08'
            >>> padding.PKCS7(_,padding.UNPAD)
            b'12345678'"""
        if direction == PAD:
            if length == None:
                raise ValueError("Supply a valid length")
            return __PKCS7(padData, length)
        elif direction == UNPAD:
            return __PKCS7_unpad(padData)
        else:
            raise ValueError("Supply a valid direction")


def __PKCS7 (toPad, length):
    amount = length - len(toPad)%length
    pattern = bytearray([amount])
    pad = pattern*amount
    return bytes(toPad + pad)

def __PKCS7_unpad (padded):
    pattern = padded[-1:]
    length = bytearray(pattern)[0]
    #check if the bytes to be removed are all the same pattern
    if padded.endswith(pattern*length):
        return padded[:-length]
    else:
        print('error: padding pattern not recognized')
        return padded

def ANSI_X923 (padData, direction, length=None):
        """Pad a string using ANSI_X923

            padData = raw string to pad/unpad
            direction = PAD or UNPAD
            length = amount of bytes the padded string should be a multiple of
                     (length variable is not used when unpadding)
            
            returns: (un)padded raw string
            
            A new block full of padding will be added when padding data that is
            already a multiple of the given length.
            
            Example:
            =========
            >>> from CryptoPlus.Util import padding
            
            >>> padding.ANSI_X923(b'12345678',padding.PAD,16)
            b'12345678\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x08'
            >>> padding.ANSI_X923(_,padding.UNPAD)
            b'12345678'"""
        if direction == PAD:
            if length == None:
                raise ValueError("Supply a valid length")
            return __ANSI_X923(padData, length)
        elif direction == UNPAD:
            return __ANSI_X923_unpad(padData)
        else:
            raise ValueError("Supply a valid direction")

def __ANSI_X923 (toPad, length):
    bytesToPad = length - len(toPad)%length
    trail = bytes(bytearray([bytesToPad]))
    pattern = b'\x00'*(bytesToPad -1) + trail
    return toPad + pattern

def __ANSI_X923_unpad (padded):
    length = bytearray(padded)[-1]
    #check if the bytes to be removed are all zero
    if padded.count(b'\x00',-length,-1) == length - 1:
        return padded[:-length]
    else:
        print('error: padding pattern not recognized %s' % padded.count('\x00',-length,-1))
        return padded

def ISO_10126 (padData, direction, length=None):
        """Pad a string using ISO_10126

            padData = raw string to pad/unpad
            direction = PAD or UNPAD
            length = amount of bytes the padded string should be a multiple of
                     (length variable is not used when unpadding)
            
            returns: (un)padded raw string
            
            A new block full of padding will be added when padding data that is
            already a multiple of the given length.
            
            Example:
            =========
            >>> from CryptoPlus.Util import padding

            >>> padded = padding.ISO_10126(b'12345678',padding.PAD,16)
            >>> padding.ISO_10126(padded,padding.UNPAD)
            b'12345678'"""
        if direction == PAD:
            if length == None:
                raise ValueError("Supply a valid length")
            return __ISO_10126(padData, length)
        elif direction == UNPAD:
            return __ISO_10126_unpad(padData)
        else:
            raise ValueError("Supply a valid direction")

def __ISO_10126 (toPad, length):
    bytesToPad = length - len(toPad)%length
    randomPattern = bytearray(random.randint(0,255) for x in range(0,bytesToPad-1))
    return bytes(toPad + randomPattern + bytearray([bytesToPad]))

def __ISO_10126_unpad (padded):
   return padded[0:len(padded)-bytearray(padded)[-1]]

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from .pysha224 import sha224

__all__ = ['new','digest_size']

def new(data=None):
        """Create a new pure python SHA-224 hash object
        
        data =  initial input (raw string) to the hashing object
                if present, the method call update(arg) is made
        
        EXAMPLE: FIPS 180-2
        =========
        
        >>> from CryptoPlus.Hash import python_SHA224
        
        >>> message = b"abc"
        >>> hasher = python_SHA224.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        '23097d223405d8228642a477bda255b32aadbce4bda0b3f7e36c9da7'
        
        >>> message = b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
        >>> hasher = python_SHA224.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        '75388b16512776cc5dba5da1fd890150b0c6455cb4f58b1952522525'
        """
        return sha224(data)
        
digest_size = sha224.digest_size



"""Util initialization

makes the Util modules from Crypto AND CryptoPlus available here
"""
from __future__ import absolute_import
#import Crypto
#from Crypto.Util import number, randpool, RFC1751
from . import padding, util, python_compat, number, randpool, RFC1751

from pkg_resources import parse_version

__all__ = ["padding","util","number","randpool","RFC1751","python_compat"]

#if parse_version(Crypto.__version__) > parse_version("2.0.1"):
#        from Crypto.Util import python_compat
#        __all__.append("python_compat")

#del Crypto



#!/usr/bin/env python

import unittest
import doctest
import re
import sys
from pkg_resources import require
require("CryptoPlus>=1.0")
#import CryptoPlus.Cipher.python_AES
from CryptoPlus.Cipher import python_AES, AES, python_DES, DES, python_DES3,\
        DES3, python_Blowfish, Blowfish, python_Twofish, python_Serpent,\
        python_Rijndael, CAST, ARC2, python_PRESENT
from CryptoPlus.Util import padding
from CryptoPlus.Hash import python_RadioGatun, python_PBKDF2, RIPEMD, python_MD5,\
     python_SHA,python_SHA256,python_SHA224,python_SHA384,python_SHA512,\
     python_whirlpool
try:
        from CryptoPlus.Cipher import IDEA
        from CryptoPlus.Cipher import RC5
        import_error = 0
except ImportError:
        import_error = 1

# from https://dirkjan.ochtman.nl/writing/2014/07/06/single-source-python-23-doctests.html
# modified for python3 to be default
class Py23DocChecker(doctest.OutputChecker):
    def check_output(self, want, got, optionflags):
        if sys.version_info[0] < 3:
            want = re.sub("b'(.*?)'", "'\\1'", want)
            want = re.sub('b"(.*?)"', '"\\1"', want)
        return doctest.OutputChecker.check_output(self, want, got, optionflags)

suite = unittest.TestSuite()
#for mod in (CryptoPlus.Cipher.python_AES,CryptoPlus.Cipher.python_AES):
for mod in python_AES, AES, python_DES, DES, python_DES3, DES3, python_Blowfish,\
           Blowfish, python_Twofish, python_Serpent, python_Rijndael, CAST, ARC2,\
           python_PRESENT, padding, python_RadioGatun, python_PBKDF2, RIPEMD,\
           python_MD5, python_SHA,python_SHA256,python_SHA224,python_SHA384,python_SHA512,\
           python_whirlpool:
    suite.addTest(doctest.DocTestSuite(mod, checker=Py23DocChecker()))
if not import_error:
    suite.addTest(doctest.DocTestSuite(IDEA, checker=Py23DocChecker()))
    suite.addTest(doctest.DocTestSuite(RC5, checker=Py23DocChecker()))
runner = unittest.TextTestRunner(verbosity=2)
runner.run(suite)




from .pysha512 import sha512

__all__ = ['new','digest_size']

def new(data=None):
        """Create a new pure python SHA-512 hash object
        
        data =  initial input (raw string) to the hashing object
                if present, the method call update(arg) is made
        
        EXAMPLE: FIPS 180-2
        =========
        
        >>> from CryptoPlus.Hash import python_SHA512
        
        >>> message = b"abc"
        >>> hasher = python_SHA512.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        'ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f'
        
        >>> message = b"abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu"
        >>> hasher = python_SHA512.new()
        >>> hasher.update(message)
        >>> hasher.hexdigest()
        '8e959b75dae313da8cf4f72814fc143f8f7779c6eb9f7fa17299aeadb6889018501d289e4900f7e4331b99dec4b5433ac7d329eeb6dd26545e96e55b874be909'
        """
        return sha512(data)
        
digest_size = sha512.digest_size



# -*- coding: utf-8 -*-
#
#  SelfTest/Util/test_number.py: Self-test for parts of the CryptoPlus.Util.number module
#
# =======================================================================
# Copyright (C) 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# =======================================================================
#

"""Self-tests for (some of) CryptoPlus.Util.number"""

__revision__ = "$Id$"

from CryptoPlus.Util.python_compat import *

import unittest

# NB: In some places, we compare tuples instead of just output values so that
# if any inputs cause a test failure, we'll be able to tell which ones.

class MiscTests(unittest.TestCase):
    def setUp(self):
        global number, math
        from CryptoPlus.Util import number
        import math

    def test_ceil_shift(self):
        """Util.number.ceil_shift"""
        self.assertRaises(AssertionError, number.ceil_shift, -1, 1)
        self.assertRaises(AssertionError, number.ceil_shift, 1, -1)

        # b = 0
        self.assertEqual(0, number.ceil_shift(0, 0))
        self.assertEqual(1, number.ceil_shift(1, 0))
        self.assertEqual(2, number.ceil_shift(2, 0))
        self.assertEqual(3, number.ceil_shift(3, 0))

        # b = 1
        self.assertEqual(0, number.ceil_shift(0, 1))
        self.assertEqual(1, number.ceil_shift(1, 1))
        self.assertEqual(1, number.ceil_shift(2, 1))
        self.assertEqual(2, number.ceil_shift(3, 1))

        # b = 2
        self.assertEqual(0, number.ceil_shift(0, 2))
        self.assertEqual(1, number.ceil_shift(1, 2))
        self.assertEqual(1, number.ceil_shift(2, 2))
        self.assertEqual(1, number.ceil_shift(3, 2))
        self.assertEqual(1, number.ceil_shift(4, 2))
        self.assertEqual(2, number.ceil_shift(5, 2))
        self.assertEqual(2, number.ceil_shift(6, 2))
        self.assertEqual(2, number.ceil_shift(7, 2))
        self.assertEqual(2, number.ceil_shift(8, 2))
        self.assertEqual(3, number.ceil_shift(9, 2))

        for b in range(3, 1+129, 3):    # 3, 6, ... , 129
            self.assertEqual(0, number.ceil_shift(0, b))

            n = 1
            while n <= 2**(b+2):
                (q, r) = divmod(n-1, 2**b)
                expected = q + int(not not r)
                self.assertEqual((n-1, b, expected),
                                 (n-1, b, number.ceil_shift(n-1, b)))

                (q, r) = divmod(n, 2**b)
                expected = q + int(not not r)
                self.assertEqual((n, b, expected),
                                 (n, b, number.ceil_shift(n, b)))

                (q, r) = divmod(n+1, 2**b)
                expected = q + int(not not r)
                self.assertEqual((n+1, b, expected),
                                 (n+1, b, number.ceil_shift(n+1, b)))

                n *= 2

    def test_ceil_div(self):
        """Util.number.ceil_div"""
        self.assertRaises(TypeError, number.ceil_div, "1", 1)
        self.assertRaises(ZeroDivisionError, number.ceil_div, 1, 0)
        self.assertRaises(ZeroDivisionError, number.ceil_div, -1, 0)

        # b = -1
        self.assertEqual(0, number.ceil_div(0, -1))
        self.assertEqual(-1, number.ceil_div(1, -1))
        self.assertEqual(-2, number.ceil_div(2, -1))
        self.assertEqual(-3, number.ceil_div(3, -1))

        # b = 1
        self.assertEqual(0, number.ceil_div(0, 1))
        self.assertEqual(1, number.ceil_div(1, 1))
        self.assertEqual(2, number.ceil_div(2, 1))
        self.assertEqual(3, number.ceil_div(3, 1))

        # b = 2
        self.assertEqual(0, number.ceil_div(0, 2))
        self.assertEqual(1, number.ceil_div(1, 2))
        self.assertEqual(1, number.ceil_div(2, 2))
        self.assertEqual(2, number.ceil_div(3, 2))
        self.assertEqual(2, number.ceil_div(4, 2))
        self.assertEqual(3, number.ceil_div(5, 2))

        # b = 3
        self.assertEqual(0, number.ceil_div(0, 3))
        self.assertEqual(1, number.ceil_div(1, 3))
        self.assertEqual(1, number.ceil_div(2, 3))
        self.assertEqual(1, number.ceil_div(3, 3))
        self.assertEqual(2, number.ceil_div(4, 3))
        self.assertEqual(2, number.ceil_div(5, 3))
        self.assertEqual(2, number.ceil_div(6, 3))
        self.assertEqual(3, number.ceil_div(7, 3))

        # b = 4
        self.assertEqual(0, number.ceil_div(0, 4))
        self.assertEqual(1, number.ceil_div(1, 4))
        self.assertEqual(1, number.ceil_div(2, 4))
        self.assertEqual(1, number.ceil_div(3, 4))
        self.assertEqual(1, number.ceil_div(4, 4))
        self.assertEqual(2, number.ceil_div(5, 4))
        self.assertEqual(2, number.ceil_div(6, 4))
        self.assertEqual(2, number.ceil_div(7, 4))
        self.assertEqual(2, number.ceil_div(8, 4))
        self.assertEqual(3, number.ceil_div(9, 4))

        # b = -4
        self.assertEqual(3, number.ceil_div(-9, -4))
        self.assertEqual(2, number.ceil_div(-8, -4))
        self.assertEqual(2, number.ceil_div(-7, -4))
        self.assertEqual(2, number.ceil_div(-6, -4))
        self.assertEqual(2, number.ceil_div(-5, -4))
        self.assertEqual(1, number.ceil_div(-4, -4))
        self.assertEqual(1, number.ceil_div(-3, -4))
        self.assertEqual(1, number.ceil_div(-2, -4))
        self.assertEqual(1, number.ceil_div(-1, -4))
        self.assertEqual(0, number.ceil_div(0, -4))
        self.assertEqual(0, number.ceil_div(1, -4))
        self.assertEqual(0, number.ceil_div(2, -4))
        self.assertEqual(0, number.ceil_div(3, -4))
        self.assertEqual(-1, number.ceil_div(4, -4))
        self.assertEqual(-1, number.ceil_div(5, -4))
        self.assertEqual(-1, number.ceil_div(6, -4))
        self.assertEqual(-1, number.ceil_div(7, -4))
        self.assertEqual(-2, number.ceil_div(8, -4))
        self.assertEqual(-2, number.ceil_div(9, -4))

    def test_exact_log2(self):
        """Util.number.exact_log2"""
        self.assertRaises(TypeError, number.exact_log2, "0")
        self.assertRaises(ValueError, number.exact_log2, -1)
        self.assertRaises(ValueError, number.exact_log2, 0)
        self.assertEqual(0, number.exact_log2(1))
        self.assertEqual(1, number.exact_log2(2))
        self.assertRaises(ValueError, number.exact_log2, 3)
        self.assertEqual(2, number.exact_log2(4))
        self.assertRaises(ValueError, number.exact_log2, 5)
        self.assertRaises(ValueError, number.exact_log2, 6)
        self.assertRaises(ValueError, number.exact_log2, 7)
        e = 3
        n = 8
        while e < 16:
            if n == 2**e:
                self.assertEqual(e, number.exact_log2(n), "expected=2**%d, n=%d" % (e, n))
                e += 1
            else:
                self.assertRaises(ValueError, number.exact_log2, n)
            n += 1

        for e in range(16, 1+64, 2):
            self.assertRaises(ValueError, number.exact_log2, 2**e-1)
            self.assertEqual(e, number.exact_log2(2**e))
            self.assertRaises(ValueError, number.exact_log2, 2**e+1)

    def test_exact_div(self):
        """Util.number.exact_div"""

        # Positive numbers
        self.assertEqual(1, number.exact_div(1, 1))
        self.assertRaises(ValueError, number.exact_div, 1, 2)
        self.assertEqual(1, number.exact_div(2, 2))
        self.assertRaises(ValueError, number.exact_div, 3, 2)
        self.assertEqual(2, number.exact_div(4, 2))

        # Negative numbers
        self.assertEqual(-1, number.exact_div(-1, 1))
        self.assertEqual(-1, number.exact_div(1, -1))
        self.assertRaises(ValueError, number.exact_div, -1, 2)
        self.assertEqual(1, number.exact_div(-2, -2))
        self.assertEqual(-2, number.exact_div(-4, 2))

        # Zero dividend
        self.assertEqual(0, number.exact_div(0, 1))
        self.assertEqual(0, number.exact_div(0, 2))

        # Zero divisor (allow_divzero == False)
        self.assertRaises(ZeroDivisionError, number.exact_div, 0, 0)
        self.assertRaises(ZeroDivisionError, number.exact_div, 1, 0)

        # Zero divisor (allow_divzero == True)
        self.assertEqual(0, number.exact_div(0, 0, allow_divzero=True))
        self.assertRaises(ValueError, number.exact_div, 1, 0, allow_divzero=True)

    def test_floor_div(self):
        """Util.number.floor_div"""
        self.assertRaises(TypeError, number.floor_div, "1", 1)
        for a in range(-10, 10):
            for b in range(-10, 10):
                if b == 0:
                    self.assertRaises(ZeroDivisionError, number.floor_div, a, b)
                else:
                    self.assertEqual((a, b, int(math.floor(float(a) / b))),
                                     (a, b, number.floor_div(a, b)))

def get_tests():
    from CryptoPlus.SelfTest.st_common import list_test_cases
    return list_test_cases(MiscTests)

if __name__ == '__main__':
    suite = lambda: unittest.TestSuite(get_tests())
    unittest.main(defaultTest='suite')

# vim:set ts=4 sw=4 sts=4 expandtab:



# -*- coding: utf-8 -*-
#
#  SelfTest/Hash/SHA256.py: Self-test for the SHA-256 hash function
#
# =======================================================================
# Copyright (C) 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# =======================================================================
#

"""Self-test suite for CryptoPlus.Hash.SHA256"""

__revision__ = "$Id$"

# Test vectors from FIPS PUB 180-2
# This is a list of (expected_result, input[, description]) tuples.
test_data = [
    # FIPS PUB 180-2, B.1 - "One-Block Message"
    ('ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad',
        'abc'),

    # FIPS PUB 180-2, B.2 - "Multi-Block Message"
    ('248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1',
        'abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq'),

    # FIPS PUB 180-2, B.3 - "Long Message"
#    ('cdc76e5c9914fb9281a1c7e284d73e67f1809a48a497200e046d39ccc7112cd0',
#        'a' * 10**6,
#         '"a" * 10**6'),

    # Test for an old PyCryptoPlus bug.
    ('f7fd017a3c721ce7ff03f3552c0813adcc48b7f33f07e5e2ba71e23ea393d103',
        'This message is precisely 55 bytes long, to test a bug.',
        'Length = 55 (mod 64)'),
]

def get_tests():
    from CryptoPlus.Hash import SHA256
    from common import make_hash_tests
    return make_hash_tests(SHA256, "SHA256", test_data)

if __name__ == '__main__':
    import unittest
    suite = lambda: unittest.TestSuite(get_tests())
    unittest.main(defaultTest='suite')

# vim:set ts=4 sw=4 sts=4 expandtab:
